#!/usr/bin/perl
# +========================================================================+
# || cat Cip-Pool Administration Tool                                     ||
# || webmin Modul                                                         ||
# || acladmin.cgi                                                         ||
# || (c) Copyright 2005 by Christian Kuelker                              ||
# || License: GPL Version 2 or later                                      ||
# +========================================================================+

package acladmin_cgi;

do '../web-lib.pl';
&init_config();

use CipUX::CiBot;
use CipUX::Cat;

# for reading the acl config file:
# for csv Cookbook Ed 3, Chapter 01, Rpt 1.15. Parsing Comma-Separated Data
use Text::ParseWords qw(parse_line);

my $BRAND    = "cipworx";
my $PROJECT  = "cipux";
my $SCRIPT   = "acladmin.cgi";
my $write    = 0; # 0 = do not write acl file, 1 = write acl file
my $acl_file = "/etc/cipux/acl_apps.conf";

&log($SCRIPT,"="x76);

&cibot_get_ldap_config();

my ($result,$role) = &cibot_acl($SCRIPT,$remote_user);
&cat_header($remote_user,\&header,\&help_search_link,\&text);

# hostname, user, arrayref
print &cat_status_line(&get_system_hostname(),
                       "$remote_user ($role) -> $result",
		       [
			&text('server'),
			&text('status'),
			$cipux{Customer},
			&text('date'),
			&text('version')
		       ]
		      );

# +=======================================================================+
# ||  CAT ACL check                                                      ||
# +=======================================================================+
if($result ne 'ok'){
  print "<h1>".&text('access_denied')."</h1>";
  my $error = &text('access_denied_explanation',
                   "<b>$remote_user</b>",
                   "<b>$role</b>");
  &log($SCRIPT,$error);
  print &cat_error($error);
  &cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});
  exit(0)
}


# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+
&log($SCRIPT,"ReadParse...");
ReadParse();# webmin

my $app = $in{'app'} if defined($in{'app'});# app = application (for changing, add status)
my $ref = $in{'ref'} if defined($in{'ref'});

&log($SCRIPT,"- got app    $app")    if defined($app);
&log($SCRIPT,"- got ref    $ref")    if defined($ref);

foreach(sort keys %in){
  &log($SCRIPT,"- IN: [$_] = [$in{$_}]") if defined($in{$_});
}

# +========================================================================+
# ||  Main                                                                ||
# +========================================================================+
print "<h2>".&text('acladmin')."</h2>";

&log($SCRIPT,"- acl file: '$acl_file'");
my %apps=();
my @acl_conf;
if(-f $acl_file){
  @acl_conf=`cat $acl_file`;
}else{
  print &cat_error(&text('acladmin_error_02',$acl_file));
}
foreach my $l  (@acl_conf){
  chomp($l);
  &log($SCRIPT,"- line: '$l'");
  if ($l=~m/^\s*[A-Za-z0-9]+/){ # line containing a char or num (after eventualy space)
    # parse_line
    # 1: Group
    # 2: Name
    # 3: path
    &log($SCRIPT,"  -> no comment ($l)");
    my @c = ();# columns
    #($c[0],$c[1]) = &parse_line(",",0, $l);
    ($c[0],$c[1]) = split(",",$l);
    foreach my $c (@c){
      $c =~ s/^\s+|\s+$//g;
      &log($SCRIPT,"    => [$c]");
      if(not defined($c)){
        &log($SCRIPT,"    => ! WARNING column not defined");
        $c="";
      }elsif($c eq ""){
	&log($SCRIPT,"    => ! WARNING column empty");
      }
    }
    $apps{$c[0]}=$c[1];
  }
}

# +========================================================================+
# ||  deregister                                                          ||
# +========================================================================+
# del app_line
if(defined($in{'deregister'})){
  &log($SCRIPT,"- delete app line: [$in{'app'}] [$in{'ref'}]");
  delete($apps{$in{'app'}});
  $write = 1;
}

# +========================================================================+
# ||  change                                                              ||
# +========================================================================+
# change_app_line
if(defined($in{'change'})){
  &log($SCRIPT,"- change app line: [$in{'app'}] [$in{'ref'}]");
  $apps{$in{'app'}}=$in{'ref'};
  $write = 1;
}

# +========================================================================+
# ||  register                                                            ||
# +========================================================================+
# add app_line
if(defined($in{'register'})){
  if(defined($in{app}) and defined($in{ref})){
    if($in{app} ne '' and $in{ref} ne ''){
      &log($SCRIPT,"- add app line: [$in{'app'}] [$in{'ref'}]");
      $apps{$in{'app'}}=$in{'ref'};
      $write = 1;
    }else{
      print &cat_error(&text('acladmin_error_01'));
    }
  }else{
    #no VALUES for VAR ref, app
    print &cat_error(&text('acladmin_error_01'));
  }
}

# +========================================================================+
# ||  change status                                                       ||
# +========================================================================+
# change status if group and app
if(defined($in{'status'})){
  my $file = $apps{$in{'app'}};
  my $group = $in{'group'};
  &log($SCRIPT,"- change status for group [$group] on app [$in{'app'}] for file [$file]");
  if(cibot_file_is_executable_for_group($file,$group)){
    # executable -> disable
    &log($SCRIPT,"-> disable");
    &cibot_disable_app($file,$group);
  }else{
    # not executable -> enable (well this mean its undefined rights = non ALC rights)
    &log($SCRIPT,"-> enable");
    &cibot_enable_app($file,$group);
  }
  $write = 0;
}

# +========================================================================+
# ||  save status                                                         ||
# +========================================================================+
if($write){
  &log($SCRIPT,"- write status");
  open(FILE,"> $acl_file") or die "Can't open file '/etc/cipux/acl_apps.conf'";
  print FILE "#acl_apps.conf\n";
  print FILE "# Do NOT edit this file: use CAT (acladmin.cgi)\n";
  print FILE "#\n";
  foreach(sort keys %apps){
    print FILE "$_,$apps{$_}\n";
    &log($SCRIPT,"- write [$_] [$apps{$_}]");
  }
  close(FILE);
}

#my $course_aref = &list_group('school_class');
my $course_href = &cibot_list('C','groupType');

print "<h2>".&text('acl_rights_on_groups')."</h2>";
print "<table border='1'>\n";
print "<tr><th $tb> &nbsp;</th>";
foreach my $app (sort keys %apps){
  if($app ne ""){
    print "<th $tb>$app</th>";
  }
}
print "</tr>";

# list only the ...
foreach my $course (sort keys %$course_href){
  if($$course_href{$course} eq "school_class"
     or $$course_href{$course} eq "school_exam"
     or $$course_href{$course} eq "guest"  ){
    print "<tr><th $tb>$course</th>\n";
    foreach my $app (sort keys %apps){
      print "<td align=center>";
      my @refs = split(' ',$apps{$app});
      foreach my $file (@refs){
	if( -f $file){
	  if(cibot_file_is_executable_for_group($file,$course)){
	    print "<a href='/cat/$SCRIPT?status=1&group=$course&app=$app' title='$file'><img src='images/r.png' alt='$file' border=0></a>";
	  }else{
	    print "<a href='/cat/$SCRIPT?status=1&group=$course&app=$app' title='$file'><img src='images/c.png' alt='$file' border=0></a>";
	  }
	}else{
	  print "<a href='/cat/$SCRIPT' title='$file'><img src='images/u.png' alt='$file' border=0></a>";
	}
      }
      print "</td>";
    }
    print "</tr>\n";
  }
}
print "</table>";

# c = red
# w = orange images/c.png <img src='images/w.png'>
# r = green

print &cat_explain_message("
<img src='images/r.png'> = ".&text('acladmin_green_status_explain')." <br>
<img src='images/c.png'> = ".&text('acladmin_red_status_explain')." <br>
<img src='images/u.png'> = ".&text('acladmin_orange_status_explain')." <br>",$cb);

print "<h2>".&text('registered_programmes')."</h2>";
print "<table border='1'>\n";
print "<tr $cb><th $tb>".&text('programme')."</th>
<th $tb>".&text('reference')."</th>
<th>".&text('action')."</th></tr>";
foreach my $app (sort keys %apps){
  print "<tr><form method=post action='$SCRIPT'>";
  # print "  <td><input type=text name='app' maxlenght=30 size=12 value='$app'></td>";
  print "  <td $tb>$app<input type=hidden name=app value=\"$app\"></td>";
  print "  <td><input type=text name='ref' maxlenght=255 size=80 value='$apps{$app}'></td>";
  print "<td $cb>
              <input type=submit value='".&text('change')."' name='change'>
              <input type=submit value='".&text('deregister')."' name='deregister'>
         </td>";
  print " </form></tr>";
}

print "</table>";

print "<h2>".&text('register_programme')."</h2>";
print "<form method=post action='$SCRIPT'>";
print &cat_table_header(2,$tb,&text('acladmin_register_programme_group'));
print &cat_table_row([1,0],[$tb,$cb],
		     [&text('acladmin_register_programme_group_name'),
		      "<input type=text name='app' maxlenght=60 size=24>"]
		    );
print &cat_table_row([1,0],[$tb,$cb],
		     [&text('acladmin_register_programme_group_reference'),
		      "<input type=text name='ref' maxlenght=255 size=80>"]
		    );
print &cat_table_footer;
print "<input type=submit value='".&text('register_programme')."' name='register'>";
print "</form>";

# +========================================================================+
# ||  Footer                                                              ||
# +========================================================================+

&cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});

exit(0);
