#!/usr/bin/perl
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || Webmin module: import.cgi                                            ||
# || This scripts imports CipUX objects: user, course                     ||
# || (c) Copyright 2001 - 2005 Christian Kuelker                          ||
# || License: GPL version 2 or later                                      ||
# +========================================================================+

package import_cgi;

do '../web-lib.pl';
&init_config();
require '../webmin/webmin-lib.pl'; # for fileimport

use CipUX::Conversion;
use CipUX::CiBot;
use CipUX::Task;
use CipUX::Cat;
use Text::ParseWords qw(parse_line); # for csv Cookbook Ed 3, Chapter 01, Rpt 1.15. Parsing Comma-Separated Data

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "import.cgi";

my $dialog = 0;
my $upload = 0;
my $error  = 0;
my @file = ();
my $file = "";
my $uquota = "";
my $gquota = "" ;
%role = ( 'student'   => 1,
	  'teacher'   => 1,
	  'surfer'    => 1,
	  'tutor'     => 1,
	  'assistent' => 1,
	  'examinee'  => 1
	);

&log($SCRIPT,"="x76);

&cibot_get_ldap_config();

my ($result,$role) = &cibot_acl($SCRIPT,$remote_user);
&cat_header($remote_user,\&header,\&help_search_link,\&text);


# hostname, user, arrayref
print &cat_status_line(&get_system_hostname(),
                       "$remote_user ($role) -> $result",
		       [
			&text('server'),
			&text('status'),
			$cipux{Customer},
			&text('date'),
			&text('version')
		       ]
		      );

# +=======================================================================+
# ||  CAT ACL check                                                      ||
# +=======================================================================+
if($result ne 'ok'){
  print "<h1>".&text('access_denied')."</h1>";
  my $error = &text('access_denied_explanation',
                   "<b>$remote_user</b>",
                   "<b>$role</b>");
  &log($SCRIPT,$error);
  print &cat_error($error);
  &cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});
  exit(0)
}

# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+

if($ENV{REQUEST_METHOD} eq "POST"){
  &log($SCRIPT,"* ReadParseMime() => upload");
  &ReadParseMime();
  $upload = 1;
}else{
  &log($SCRIPT,"* ReadParse() => no upload");
  &ReadParse();
  $upload = 0;
}

my $action    = $in{'action'}    if defined($in{'action'});# action

foreach(sort keys %in){
  &log($SCRIPT,"- IN: [$_] = [$in{$_}]") if defined($in{$_});
}

if($in{mode} eq "add"){
  %mandatory = (
		cipuxFirstname => 'Vorname',
		cipuxLastname  => 'Nachname',
		course         => 'Klasse/Kurs/Seminar',
	     );
}else{
  %mandatory = (
		uid            => 'Kennung',
	       );
}

# =========================================================================+
#                    Group List                                           ||
# =========================================================================+

my $group_href = &cibot_list('C','gidNumber');
my $group_type_href = &cibot_list('C','groupType');

# =========================================================================+
#                           ERROR CHECKS                                  ||
# =========================================================================+

# check for both given files
if(defined($in{upload}) and defined($in{file})){
  if($in{upload} ne '' and $in{file} ne ''){ #both files are given!
    &log($SCRIPT,"# ERROR: use only one file, please!");
    &cat_print_html_error(1,&text('importadmin_error_01'));
    #  &cat_error_info_dialog($SCRIPT,"Zwei Dateien erhalten!","Es ist nur m&ouml;glich jeweils eine Datei auf einmal zu importieren. Bitte entscheiden sie sich f&uuml;r nur <b>eine</b> Datei!",'dummy','dummy','dummy','dummy');
    $error = 1;
  }
}

# check for valid file
if(defined($in{file})){
  if($in{file} ne ''){ # server file
    $file = $in{file};
    &log($SCRIPT,"- 1 file [$file]");
    if(!(-r $file)){
      &log($SCRIPT,"- 1 WARNING [$file] not readable!");
      &cat_error(&text('importadmin_error_02'));
      $error = 1;
    }else{
      (@file) = `cat $file`;
    }
  }
}

#check vor valid upload
if(defined($in{upload_filename})){
  if($in{upload_filename} ne ''){ # upload file
    &log($SCRIPT,"- 1 got upload");
    if($in{upload} eq ''){
      &log($SCRIPT,"- 1 WARNING upload empty!");
      &cat_error(&text('importadmin_error_03'));
      $error = 1;
    }else{
      (@file) = split("\n",$in{upload});
    }
  }
}

# =========================================================================+
#                           IMPORT 1                                      ||
# =========================================================================+

if(defined($in{import})){
  &log($SCRIPT,"+ 1 import");
  $dialog = 1;

  if( not $error){
    print "<h2>".&text('import')."</h2>\n";

    my $group_aref = &list_group('school_class');
    my $groups = "<select name='defaultgroup' size=1>\n<option>";
    $groups .= join("</option>\n<option>",@$group_aref);
    $groups .= "</option>\n</select>\n";

    print "<form enctype=multipart/form-data method=post action='$SCRIPT'>\n";
    print &cat_explain_message(&text('importadmin_explain_01',$groups),$cb);
    print "<input type=radio checked name=mode value='add'>";
    print &text('importadmin_add_objects');
    print "<input type=radio name=mode value='del'>";
    print &text('importadmin_delete_objects');
    print &cat_table_header(100,$tb,&text('importadmin_file_content').":");
    my @line =();
    foreach my $l (@file){ # for each line $l
      chomp($l);
      &log($SCRIPT,"- 1 line [$l]");
      if($l =~ m/^#/ or $l eq ""){
	&log($SCRIPT,"- 1 will not use this  line [$l]");
      }else{
	&log($SCRIPT,"- 1 use this line [$l]");
	(@line) = &parse_line(",",0, $l);
	# TODO: if you want to edit the stuff in advance, unkomment this and
	#       make a matrix conversion (col->row) in DO IMPORT 2
	#my $z =0;
	#foreach my $c (@line){
	#  $c = "<input type=text value='$c' name='col$z' maxlenght=255 size=15>\n";
	# $z++;
	#}
	print &cat_table_row([0],[''],\@line);
      }
    }
    my @col=();
    my $z =0;
    foreach my $l (@line){
      push(@col, &cat_choose_column_col($z));
      $z++;
    }
    print &cat_table_row([0],[''],\@col);
    print &cat_table_footer;
    print "<input type=hidden name=col value='$z'>\n";
    print "<input type=hidden name=file value='$in{file}'>\n";
    print "<input type=hidden name=upload_filename value='$in{upload_filename}'>\n";
    print "<input type=hidden name=upload value='$in{upload}'>\n";
    print "<input type=submit value='ok' name='do_import'>\n";
    print "<input type=submit value='".&text('chancel')."' name='chancel'>\n";
    print "</form>";
    print &cat_explain_message(&text('importadmin_explain_02'),$cb);
  }
}

# =========================================================================+
#                           DO IMPORT 2                                   ||
# =========================================================================+
if(defined($in{do_import})){
  &log($SCRIPT,"+ 2 do_import");
  if(not $error){
    &log($SCRIPT,"- no error");
    $dialog = 1;
    my $col = $in{col}-1;
    my @col = ();
    print "<h2>".&text('do_import')."</h2>";
    print &cat_explain_message(&text('importadmin_explain_03'),$cb);

   for (my $c=0; $c <= $col; $c++){
     push(@col,$in{"head$c"});
    # print "<li>$in{"head$c"}</li>\n";
   }
#    print "<ul><li> importiere aus <b>".($col+1)."</b> Spalten ...</li>\n";
#    print "</ul>\n";

    my @line =();
    my $group_type_href = &cibot_list('C','groupType');
    my $user_role_href = &cibot_list('U','cipuxRole');

    foreach my $l (@file){
      chomp($l);
      &log($SCRIPT,"- 2 line [$l]");
      if($l =~ m/^#/ or $l eq ""){
	&log($SCRIPT,"- 2 will not use this  line [$l]");
      }elsif($l =~ m/,/){
	&log($SCRIPT,"- 2 use this line [$l]");
	(@line) = &parse_line(",",0, $l);
	my $z = 0;
	%data=();
	foreach my $c (@col){
	  &log($SCRIPT,"- 2 col [$c] line [$line[$z]]");
	  $data{$c} = $line[$z];
	  $z++;
	}
	$data{cipuxFirstname}=&conversion_name($data{cipuxFirstname});
	$data{cipuxLastname}=&conversion_name($data{cipuxLastname});
	$data{password} = &cibot_random_password() if not defined($data{password});
	$data{password} = &cibot_random_password() if $data{password} eq '';
	$data{password} = &conversion_password($data{password});
	if($in{mode} eq "add"){# not for del! (it will iterate +1, and we will be never right)
	  $data{uid}= &cibot_get_login($data{cipuxFirstname},$data{cipuxLastname},$user_role_href) if not defined($data{uid});
	  $data{uid}= &cibot_get_login($data{cipuxFirstname},$data{cipuxLastname},$user_role_href) if $data{uid} eq '';
	}
	$data{uid}=&conversion_login($data{uid});
	if(defined($data{course})){
	  $data{course}=&conversion_login($data{course});
	}else{
	  $data{course}=&conversion_login($in{defaultgroup});
	}
        &log($SCRIPT,"- cipuxRole befor [$data{cipuxRole}]") if defined($data{cipuxRole});
	$data{cipuxRole} =~ s/^\s+//g;
	$data{cipuxRole} =~ s/\s+$//g;
        &log($SCRIPT,"- cipuxRole after [$data{cipuxRole}]") if defined($data{cipuxRole});
	$data{cipuxRole}="student" if not defined($role{$data{cipuxRole}});
	if(defined($data{cipuxGroupQuota})){
	  $gquota = &conversion_integer($data{cipuxGroupQuota});
	}else{
	  $gquota= &conversion_integer($ldap{CipUX}{Cipux_Default_Group_Hard_Quota});
	}
	&log($SCRIPT,"- gquota [$gquota] [$ldap{CipUX}{Cipux_Default_Group_Hard_Quota}]");
	if(defined($data{cipuxUserQuota})){
	  $uquota = &conversion_integer($data{cipuxUserQuota});
	}else{
	  $uquota= &conversion_integer($ldap{CipUX}{Cipux_Default_User_Hard_Quota});
	}
	&log($SCRIPT,"- uquota [$uquota] [$ldap{CipUX}{Cipux_Default_User_Hard_Quota}]");
	# test for errors:
	foreach my $obj (keys %mandatory){
	  # import.cgi: - 2 use this line [klasse5a,student,bbeutlin,Beutlin,Bilbo,oadfldkf]
	  # import.cgi: - test for obj [course] data [klasse84] mandatory [Klasse/Kurs/Seminar]
	  # import.cgi: - test for obj [password] data [] mandatory [Passwort (Klartext)]
	  &log($SCRIPT,"- test mandatory [$obj] => error if(data [$data{$obj}] eq '') => error with [$mandatory{$obj}] ");
	  &local_error($mandatory{$obj}) if $data{$obj} eq '';
	}
	print "<ul>";
	if($in{mode} eq "add"){

	  &log($SCRIPT," - test for: [$data{course}] eq [$$group_type_href{$data{course}}]");
	  if(not defined($$group_type_href{$data{course}})){
	    # add course
	    &log($SCRIPT,"- add extra course [$data{course}]");
	    &log($SCRIPT,"- create login course [$data{course}]");
	    print "<li>".&text('importadmin_add_group_login',$data{course})."</li>";
	    &cipux_task_create_course('cgi',
				     $data{course},
				     $data{password},
				     $gquota,
				     &cibot_auto_softquota($gquota)
				     );
	    &log($SCRIPT,"- create login user [$data{course}]");
	    print "<li>".&text('create_login',$data{course})."</li>";
	    $group_type_href = &cibot_list('C','groupType');
	    $user_role_href = &cibot_list('U','cipuxRole');
	  }else{
	    &log($SCRIPT,"- the course [$data{course}] already exists!");
	    print "<li>".&text('group_already_exist',$data{course})."</li>";
	  }

	  &log($SCRIPT," - test for private user group uid: [$data{uid}] eq [$$group_type_href{$data{uid}}]");
	  if(not defined($$group_type_href{$data{uid}})){
	    &log($SCRIPT,"- add private user group [$data{uid}]");
	    # add private group
	    print "<li>".&text('importadmin_add_private_group',$data{uid})."</li>";

	    my $cmd = "/usr/bin/cipux_add -c -w '$data{password}'"
	      ." --attribute cn='$data{uid}'"
	      ." --attribute cipuxRole='$data{cipuxRole}'"
	      ." --attribute groupType=private"
	      ." --attribute cipuxSoftQuota='".&cibot_auto_softquota($gquota)."'"
	      ." --attribute cipuxHardQuota='$gquota'"
	      ." --attribute memberUid='$data{uid}'";
	    # &log($SCRIPT," - cmd [$cmd]");
	  &cibot_exec_cmd('html',$cmd,'');
	  }else{
	    &log($SCRIPT,"- the private group [$data{uid}] already exists!");
	    print "<li>".&text('private_group_already_exist',$data{uid})."</li>";
	  }
	  if(not defined($$user_role_href{$data{uid}})){
	    &log($SCRIPT,"- add user [$data{uid}]");
	    # add user
	    print "<li>".&text('importadmin_add_user_login',$data{uid})."</li>";
	    $cmd = "/usr/bin/cipux_add -u -w '$data{password}'"
	      ." --attribute cipuxFirstname='$data{cipuxFirstname}'"
	      ." --attribute cipuxLastname='$data{cipuxLastname}'"
	      ." --attribute uid='$data{uid}'"
	      ." --attribute cipuxRole='$data{cipuxRole}'"
	      ." --attribute cipuxHardQuota='$uquota'"
              ." --attribute cipuxSoftQuota='".&cibot_auto_softquota($uquota)."'";
	    &cibot_exec_cmd('html',$cmd,'');
	    print "<li>".&text('add_login_to_group',$data{uid},$data{course})."</li>";
	    &cipux_task_add_user_to_course('cgi',$data{course},$data{uid});
	    if(defined($data{cipuxRole})){
	      print "<li>".&text('add_login_to_group',$data{uid},$data{cipuxRole})."</li>";
	      &cipux_task_add_user_to_course('cgi',$data{cipuxRole},$data{uid});
	    }else{
	      # Remark: this will never happen, because if its not defined, it will be student
	      print "<li>".&text('importadmin_the_role_did_not_exist_do_import_again',$data{cipuxRole})."</li>";
	    }
	}else{
	  &log($SCRIPT,"- the user [$data{uid}] already exists!");
	  print "<li>".&text('login_already_exist',$data{uid})."</li>";
	}
	}elsif($in{mode} eq "del"){
	  &log($SCRIPT,"- del uid[$data{uid}]");
          if(defined($$user_role_href{$data{uid}})){
	    # uid exist, delete
	    &cipux_task_delete_user('cgi',$data{uid});
	    &log($SCRIPT,"- uid exist: delete group [$data{uid}] ");
	    print "<li>".&text('delete_group',$data{uid})."</li>";
	    &log($SCRIPT,"- uid exist: delete user [$data{uid}] ");
	    print "<li>".&text('delete_user_login',$data{uid})."</li>";
	  }else{
	    # uid do not exist, no delete
	    &log($SCRIPT,"- uid do not exist: no delete");
	    print "<li>".&text('importadmin_nonexistend_login',$data{uid})."</li>";
	  }
	}
	print "</ul>";
      }
    }
  }else{
      # error file not redable
    &log($SCRIPT,"- 2 WARNING [$file] not readable!");
    &cat_error(&text('importadmin_error_02'));
  }

  print &cat_explain_message(&text('importadmin_explain_04'),$cb);
}

# =========================================================================+
#                           MAIN                                          ||
# =========================================================================+

if(not $dialog){
  &log($SCRIPT,"- main");

  print "<h2>".&text('import')."</h2>\n";
  print "<h3>".&text('importadmin_import_objects')."</h3>\n";

  print "<form action='import.cgi' accept='text/*' enctype=multipart/form-data method=post>\n";
  print &cat_table_header(2,$tb,&text('importadmin_import_file'));
  print &cat_table_row([1,0],
		       [$tb,$cb],
		       [&text('importadmin_import_server_file'),"<input name=file size=50>".&file_chooser_button("file", 0)]);
  print &cat_table_row([1,0],
		       [$tb,$cb],
		       [&text('importadmin_import_browser_file'),"<input name=upload type=file size=50><br>\n"]);
print &cat_table_footer;
print "<input type=submit name=import value='".&text('do_import')."'>";
print "</form>";
# print '<input name=file size=50>\n;<input type=button onClick=\'ifield = document.forms[0].file; chooser = window.open("chooser.cgi?type=0&chroot=/&file="+escape(ifield.value), "chooser", "toolbar=no,menubar=no,scrollbar=no,width=400,height=300"); chooser.ifield = ifield; window.ifield = ifield\' value="suchen">'
#print ,"<br>\n";

  print &cat_explain_message(&text('importadmin_explain_05'),$cb);
}
# +=======================================================================+
# || Footer                                                              ||
# +=======================================================================+

&cat_footer($remote_user,\&footer);

exit(0);

sub local_error{
  my $col = shift;
  &cat_print_html_error(4,&text('importadmin_error_04',$col));
  &cat_footer($remote_user,\&footer);
  exit(1);
}
