#!/usr/bin/perl
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || Webmin module: skeladmin.cgi                                         ||
# || Administrate user accounts                                           ||
# || (c) 2001 - 2005 Christian Kuelker                                    ||
# || License: GPL version 2 or later                                      ||
# +========================================================================+

package skeladmin_cgi;

do '../web-lib.pl';
&init_config();

use CipUX::CiBot;
use CipUX::Conversion;
use CipUX::Cat;
use CipUX::Task;

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "skeladmin.cgi";

my $quotatype       = 0; # userquota
my $reread_userlist = 0; # to be reread if new user
my $dialog          = 0;
my @skel            = ();
my @course          = ();

&log($SCRIPT,"="x76);

&cibot_get_ldap_config();

my ($result,$role) = &cibot_acl($SCRIPT,$remote_user);
&cat_header($remote_user,\&header,\&help_search_link,\&text);


# hostname, user, arrayref
print &cat_status_line(&get_system_hostname(),
                       "$remote_user ($role) -> $result",
		       [
			&text('server'),
			&text('status'),
			$cipux{Customer},
			&text('date'),
			&text('version')
		       ]
		      );

# +=======================================================================+
# ||  CAT ACL check                                                      ||
# +=======================================================================+
if($result ne 'ok'){
  print "<h1>".&text('access_denied')."</h1>";
  my $error = &text('access_denied_explanation',
                   "<b>$remote_user</b>",
                   "<b>$role</b>");
  &log($SCRIPT,$error);
  print &cat_error($error);
  &cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});
  exit(0)
}

# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+
&log($SCRIPT,"- ReadParse...");
ReadParse();# webmin

$size = 0;
my $letter    = $in{'letter'} if defined($in{'letter'});
my $action    = $in{'action'} if defined($in{'action'});# action
my $user      = $in{'user'}   if defined($in{'user'});# action

my $uidNumber = $in{'uidNumber'} if defined($in{'uidNumber'});# action = set

foreach(sort keys %in){
  &log($SCRIPT,"- IN: [$_] = [$in{$_}]") if defined($in{$_});
}

# =========================================================================+
# ||               WARNING delete user (question)                         ||
# =========================================================================+
if($user ne "" and defined($in{skeladmin_delete_user})){
  $dialog=1;
  &log($SCRIPT,"- skeladmin_delete_user [$user] ");
  # realy want to delete this user?
  &cat_warn_dialog($SCRIPT,
		   &text('skeladmin_delete'),
		   &text('skeladmin_delete_question',$user),
		   $user,$crole,'skeladmin_delete_user_ok',
		   &text('chancel'),$letter);
}

# =========================================================================+
# ||                Delete user (execution)                               ||
# =========================================================================+
if($user ne "" and defined($in{skeladmin_delete_user_ok})){
  &log($SCRIPT,"@ skeladmin_delete_user, user $user");
  print "<ul><li>".&text('delete_group',$user)."</li>";
  &cipux_task_delete_user('cgi',$user);
  print "</ul>";
}


# =========================================================================+
# ||               Add new user (student, theacher)                       ||
# =========================================================================+
if(defined($in{create_user})){
  # skeladmin.cgi: - IN: [create_user] = [Benutzer hinzufgen]
  # skeladmin.cgi: - IN: [login] = [LOGIN]
  # skeladmin.cgi: - IN: [password] = [PASS]

  my @args = qw (login password);

  my $error = "";
  foreach (@args){
    &log($SCRIPT,"- want arg [$_] with non empty value [$in{$_}]");
    &log($SCRIPT,"  -> ok [$_] is defined") if defined($in{$_});
    if($in{$_} eq ""){
      $error = &text('skeladmin_error_01',&text($_));
      # $error = "Bitte geben sie einen Wert f&uuml;r <b>'".&text('$_')."'</b> an!" ;
      &log($SCRIPT,"* ".$error);
      print &cat_error($error);
    }else{
      &log($SCRIPT,"  -> ok [$_] is not empty");
    }
  }
  if($error eq ""){
    my $login     = &conversion_login($in{login});
    my $password  = &conversion_password($in{password});
    if($password eq ''){
      # ERROR empty password
      print &cat_print_html_error(12,&text('skeladmin_error_12'));
    }else{

      print "<li>".&text('create_login',$login)."</li>";

      &cipux_task_create_skel('cgi',$login,$password);

    }
    $reread_userlist = 1;
  }
}

# =========================================================================+
#                    User List                                            ||
# =========================================================================+

my $user_href  = &cibot_list('U','uidNumber');# list all user-nodes's uidNumber
my $urole_href = &cibot_list('U','cipuxRole');# list all user-nodes's role
my $crole_href = &cibot_list('C','cipuxRole');# list all group-nodes's role

foreach my $login (sort keys %$user_href){

  &log($SCRIPT, "- login [$login] urole [$$urole_href{$login}] crole [$$crole_href{$login}]");

  #if($$urole_href{$login} eq 'account'
  #  and $$crole_href{$login} eq 'skel'){
  if($$crole_href{$login} eq 'skel'){

    push(@login,$login);
    &log($SCRIPT,"- [$login] eq account/skel => add $login to \@login");
  }
}

# =========================================================================+
# | =                    "Change password";#                            ||
# =========================================================================+
if(defined($in{submit_password})){
  &log($SCRIPT,"* submit_password");
  my $password = &conversion_password($in{pwd});
  if($password ne ""){
    &log($SCRIPT,"- password is NOT empty");
    # test for non asci chars!
    if($password =~ m/_/ or $password =~ m/_+/ or $password =~ m/\W+/ or $password =~ m/'+/){
      &cat_error_info_dialog($SCRIPT,
			     &text('passwort_problem_01'),
			     &text('passwort_problem_02'),
			     ,$user,$crole,'dummy',$letter);
      $dialog=1;
    }else{
      &log($SCRIPT,"- password contain probably no bad chars");
      print "<ul><li>".&text('skeladmin_set_password_for_login_to',
			     $user,$password)."</li></ul>";
      &cipux_task_change_user_password('cgi',$user,$password);
    }
  }else{
      &cat_error_info_dialog($SCRIPT,
			     &text('passwort_problem_01'),
			     &text('passwort_problem_03'),
			     $user,$crole,'dummy',$letter);
      $dialog=1;
  }
}

# +========================================================================+
# ||  Main                                                                ||
# +========================================================================+
if($dialog){
  # reset or restore, ....

}else{
  # =========================================================================+
  # ||                        PRINT USER LIST                               ||
  # =========================================================================+

  print "<h2>".&text('skeladmin')."</h2>";

  &log($SCRIPT,"print userlist ...");

  my %remark = (
    'role' => &text('skeladmin_known_user_role'),
    'meta' => &text('skeladmin_known_user_meta'),
    'account' => &text('skeladmin_known_user_account'),
  );

  print &cat_table_header(4,$tb,&text('skeladmin_known_user'));
  print &cat_table_row([1,1,1,1],
		       [$tb,$tb,$tb,$tb,$tb],
		       [&text('login'),
			&text('role'),
			&text('password'),
			&text('login')]);

  foreach my $u (@login){

    # prepare col values:
    my $delete = "&nbsp; [$$urole_href{$u} ]";
    if($$urole_href{$u} eq 'account'){
      $delete = "<input type=submit name=skeladmin_delete_user value='".&text('delete')."'>";
    }else{
      $delete = &text('default');
    }
    my $remark = $remark{$$urole_href{$u}};

    # print row:
    print "<form method=post action='skeladmin.cgi'>
      <input type=hidden name=user value=\"$u\" >";
    print &cat_table_row(
	  	       [1,0,0,0],
			 [$tb,'','',''],
			 [
			  $u,
			  $remark,
			  &cat_open_passwd_col($SCRIPT,'userPassword',''),
			  $delete
			 ]);
     print "</form>";

  }

  print &cat_table_footer;


   foreach my $r (@roles){
   print &skel_list($r);
  }

  &print_add_user(&text('add_new_object',&text('skel')));

  print &cat_explain_message(&text('skeladmin_explain_01'));

}

# +=======================================================================+
# || Footer                                                              ||
# +=======================================================================+

&cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});

exit(0);

# +========================================================================+
# || print_add_user                                                       ||
# +========================================================================+
sub print_add_user{
  my $title = shift;

  print "<br><form method=post action='$SCRIPT'>";
  print &cat_table_header(2,$tb,$title);
  print &cat_table_row([1,0],
		       [$tb,$cb],
		       [&text('login'),"<input type=text name='login' maxlenght=20 size=24>"]);
  print &cat_table_row([1,0],
		       [$tb,$cb],
		       [&text('password'),"<input type=text name='password' maxlenght=60 size=24>"]);
  print &cat_table_footer;
  print "<input type=submit value='".&text('add_account')."' name='create_user'></form>";
}


