#!/usr/bin/perl
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || Webmin module: tutoradmin.cgi                                        ||
# ||                                                                      ||
# || (c) 2001,2002,2003,2005 Christian Kuelker                            ||
# || License: GPL version 2 or later                                      ||
# +========================================================================+

package tutoradmin_cgi;

do '../web-lib.pl';
&init_config();

use CipUX::CiBot;
use CipUX::Conversion;
use CipUX::Cat;
use CipUX::Task;

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "tutoradmin.cgi";

#my $explain = "#1a9102";
#my $red     = "#ff0000";
#my $blue    = "#0000ff";
#my $green   = "#136302";
#my $warning = "#ee2222";

&log($SCRIPT,"="x76);

&cibot_get_ldap_config();

my ($result,$role) = &cibot_acl($SCRIPT,$remote_user);
&cat_header($remote_user,\&header,\&help_search_link,\&text);


# hostname, user, arrayref
print &cat_status_line(&get_system_hostname(),
                       "$remote_user ($role) -> $result",
		       [
			&text('server'),
			&text('status'),
			$cipux{Customer},
			&text('date'),
			&text('version')
		       ]
		      );

# +=======================================================================+
# ||  CAT ACL check                                                      ||
# +=======================================================================+
if($result ne 'ok'){
  print "<h1>".&text('access_denied')."</h1>";
  my $error = &text('access_denied_explanation',
                   "<b>$remote_user</b>",
                   "<b>$role</b>");
  &log($SCRIPT,$error);
  print &cat_error($error);
  &cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});
  exit(0)
}

# some other global vars:
my @new_member     = ();
my $user_role_href = '';

# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+
&log($SCRIPT,"- ReadParse...");
ReadParse();# webmin

my $group   = $in{'group'}  if defined($in{'group'});
my $action  = $in{'action'} if defined($in{'action'});

foreach my $key (keys %in){
  &log($SCRIPT,"IN: ($key): '".$in{$key}."'<br>");
  push(@new_member,$in{$key}) if $key=~m/^1_/;#workaround, how parsing: a^@b^@c ?
  if($in{$key} eq ''){
    $fehler=1;
    print "<font color=#ff0000>$text{'application_error'} (1): $text{$key}</font><br>";
    print "<b>$text{'groupmember_empty_field'}</b><br>";
  }
}

# +=======================================================================+
# || Main                                                                ||
# +=======================================================================+

# .-----------------------------------------------------------------------.
# | Add member                                                            |
# .-----------------------------------------------------------------------.
if($action eq "$text{'tutoradmin_confirm_add'}"){
  if($group ne ""){
    &log($SCRIPT,"[ADD MEMBER]");
    &log($SCRIPT,"GROUP $group<br>");
    foreach(@new_member){
      chomp();
    }
    my $member = join("\t",@new_member);

    &cipux_task_add_user_to_course('cgi',$group,$member);

    print "$text{'tutoradmin_add_new_member'} <b>$member</b><br>";
    &log($SCRIPT,"NEW member: $member");
  }
  $action="";
}

# .-----------------------------------------------------------------------.
# | Del Member                                                            |
# .-----------------------------------------------------------------------.
if($action eq "$text{'tutoradmin_confirm_del'}"){
  if($group ne ""){
  &log($SCRIPT,"[DEL MEMBER]");
    foreach(@new_member){
      chomp();
    }
    my $member = join("\t",@new_member);
    &cipux_task_remove_user_from_course('cgi',$group,$member);
    print "$text{'tutoradmin_del_new_member'} <b>$member</b><br>";
    &log($SCRIPT,"NEW member: $member");
  }
  $action="";
}

# .-----------------------------------------------------------------------.
# | Group list                                                            |
# .-----------------------------------------------------------------------.
if($action eq ""){
  &log($SCRIPT,"[GROUP LIST]");
  my $cmd = "cipux_task_list_courses";
  &log($SCRIPT,"CMD: '$cmd'");
  my @groups=`$cmd`;
  my $course_type_href = &cibot_list('C','groupType');#

  print "<h2>$text{'tutoradmin_title_group_list'}</h2>";
  print "<table border=1>";
  print "<tr $tb><th>$text{'tutoradmin_group'}</th><th>$text{'tutoradmin_groupmembers'}</th></tr>";
  foreach(@groups){
    chomp();
    &log($SCRIPT,"GROUP $_");
    if($$course_type_href{$_} ne 'private'){
      print "<tr $cb><td>";
      print "$_</td><td>";
      print "<form method=post action='tutoradmin.cgi'>";
      print " <input type=hidden value='$_'                  name='group'>";
      print " <input type=submit value='$text{'tutoradmin_list'}' name='action' >";
      print " <input type=submit value='$text{'tutoradmin_add'}'  name='action' >";
      print " <input type=submit value='$text{'tutoradmin_del'}'  name='action' >";
      print "</form>";
      print "</td></tr>";
    }
  }
  print "</table>";
}

# .-----------------------------------------------------------------------.
# | Member list                                                           |
# .-----------------------------------------------------------------------.

if( ($action eq "$text{'tutoradmin_list'}"
     or $action eq "$text{'tutoradmin_add'}"
     or $action eq "$text{'tutoradmin_del'}"
    ) and $group ne ""){
  &log($SCRIPT,"[MEMBER LIST]");
  $user_role_href = &cibot_list('C','cipuxRole');
  print "<h2>";
  print $text{'tutoradmin_title_member_list'} if $action eq $text{'tutoradmin_list'};
  print $text{'tutoradmin_title_member_add'} if $action eq $text{'tutoradmin_add'};
  print $text{'tutoradmin_title_member_del'} if $action eq $text{'tutoradmin_del'};
  print " '$group':";
  print "</h2>";

  my @members = &cipux_task_list_user_members_of_course('cgi',$group);

  &log($SCRIPT," - action [$action] ");
  &log($SCRIPT," - add [$text{'tutoradmin_add'}] ");
  &log($SCRIPT," - add [$text{'tutoradmin_del'}] ");
  &log($SCRIPT," - add [$text{'tutoradmin_list'}] ");
  &log($SCRIPT," - group [$group] ");
  &log($SCRIPT," - members [$#members]");


  if($action eq "$text{'tutoradmin_add'}"){
    # calculate every A (all users) which is not in B (existing members)
    # chookbook 4.7 <110>
    my @seen=();
    @seen{@members}=();
    @all = &cipux_task_list_users('cgi');
    my %seen=();
    @seen{@members}=();
    @members=();
    foreach(@all){
      push(@members,$_) unless exists $seen{$_};
    }
  }
  print "<form method=post action='tutoradmin.cgi'>";
  print "<table border=1>";
  print "<tr $tb><th>";
  print $text{'tutoradmin_title_member_list'} if $action eq $text{'tutoradmin_list'};
  print $text{'tutoradmin_title_member_add'} if $action eq $text{'tutoradmin_add'};
  print $text{'tutoradmin_title_member_del'} if $action eq $text{'tutoradmin_del'};
  print " '$group': ";
  #print " $action: " if $action ne $text{'tutoradmin_list'};
  print "</th></tr>";
  my $user_fristname_href = &cibot_list('U','cipuxFirstname');
  my $user_lastname_href  = &cibot_list('U','cipuxLastname');
  foreach(sort @members){
    chomp();
    if($$user_role_href{$_} eq 'student'
       or $$user_role_href{$_} eq 'guest'
       or $$user_role_href{$_} eq 'teacher' ){
      print "<tr $cb><td>";
      chomp($group);
      if($_ ne $group){
	print "<input type=checkbox value='$_' name='1_$_'>\n" if $action ne $text{'tutoradmin_list'};
      }
      print "<b>$_</b>: $$user_fristname_href{$_} $$user_lastname_href{$_} ($text{$$user_role_href{$_}})</td>";
    }
  }
  print "</table>";
  if($action eq $text{'tutoradmin_add'}){
    print "<br><br><input type=submit value='$text{'tutoradmin_confirm_add'}'name=action >";
  }
  if($action eq $text{'tutoradmin_del'}){
    print "<br><br><input type=submit value='$text{'tutoradmin_confirm_del'}'name=action >";
  }
  print "<input type=hidden value='$group' name='group'>" if $action ne $text{'tutoradmin_list'};
  print "</form>";

}
print "<br>";
print "<img src='/cat/images/left.png' border=0>";
print " <a href='tutoradmin.cgi'>".$text{back_to_tutoradmin}."</a>";
print "<br>";
print "<table width=40% cellpadding=5 cellspacing=5 border=0 $cb>";
print "<tr $cb><td><font color=$explain>$text{'tutoradmin_logout_hint'}</font></td></tr>";
print "</table>";

# +=======================================================================+
# || Footer                                                              ||
# +=======================================================================+

&cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});

exit(0);
