# +========================================================================+
# || Cat.pm                                                               ||
# || A simple Module for Webmin CAT                                       ||
# ||                                                                      ||
# || (c) 2001,2002,2003,2004,2005 Christian Kuelker                       ||
# || License: GPL version 2 or later                                      ||
# +========================================================================+

package CipUX::Cat;
require 5.008000;

# +========================================================================+
# || Globale VARS (no Export)                                             ||
# +========================================================================+

use strict;
use vars  qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS %text);
#use vars  qw(@ISA @EXPORT %EXPORT_TAGS);
#use Date::Calc qw(Delta_Days Today Day_of_Week);
use Quota;

require Exporter;
our @ISA=qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(&cat_header
	     &cat_status_line
	     &cat_abc_nav &cat_footer
	     &cat_explain_message
	     &cat_error
	     &cat_print_html_error
	     &cat_table_header
	     &cat_table_row
	     &cat_table_footer
	     &cat_open_passwd_col
	     &cat_passwd_col
	     &cat_del_col
	     &cat_status_col
	     &cat_quota_usage_cols
	     &cat_quota_set_col
	     &cat_cipux_variable_col
	     &cat_button_col
	     &cat_choose_column_col
	     &cat_action_status
	     &cat_warn_dialog
	     &cat_error_info_dialog
	     &cat_confirm_dialog
	     $red $blue $green $orange $dark_orange $warning $explain);

our $VERSION = '3.002000';


=head1 NAME

CipUX::Cat - Perl extension for CipUX

=head1 SYNOPSIS

  use CipUX::Cat;

=head1 DESCRIPTION

This module provides library functions for serveral
perl cgi scripts from the CAT family of
the CipUX project, as a part of CipWorX.

B<Features:>

=over

=item * user administration

=item * group administration

=back

=head2 EXPORT

TODO


=head1 REQUIREMENTS

Configuration Files in /etc/cipux/cat


Perl modules:

CGI::Carp
CipUX::CiBot
Quota

=head1 USAGE

Perl CGI scripts witch would like to use Cat.pm should have lines
like the following:

  use CipUX::Cat;

=head1 FUNCTION

The perl module Cat.pm provides the following functions:

 * TODO

=cut



# +========================================================================+
# || Global  VARS (to be exported)                                        ||
# +========================================================================+
our $red         = "#ff0000";
our $blue        = "#0000ff";
our $green       = "#136302";
our $orange      = "#FFA500";
our $dark_orange = "#FF8C00";
our $warning     = "#ee2222";
our $explain     = "#1a9102";

my $Customer = "skolelinux";

# +========================================================================+
# || cat_header (a webmin header)                                         ||
# +========================================================================+
sub cat_header{
  my $remote_user = shift;
  local *header = shift;# webmin function with the same name
  local *help_search_link = shift;# webmin function with the same name
  local *text = shift; # webmin function with the same name
  my $link = '<a href="/session_login.cgi?logout=1">'.&text('logout').'</a><br>
              <a href="http://www.skolelinux.de/">Skolelinux</a> <br>
              <a href="http://www.cipux.org/">CipUX</a> |
              <a href="http://www.cipworx.org">CipWorX</a>';
  if($remote_user eq "root" or $remote_user eq "admin"){
    &header(
	    "CAT", # title
	    "",                   # image
	    "help/index.html",    # left: help
	    0,                    # left: config
	    0,                    # left: noindex      /cat
	    0,                    # left: noweminindex /
	    &help_search_link("CipUX","man") . $link
	   );
  }else{
    &header(
	    "CAT", # title
	    '',                   # image
	    undef(),              # left: help
	    '',                   # lefxt: config
	    'noindex',     # lefxt: noindex
	    'noroot',             # left: noroot
	     $link
	   );
  }
}
# +========================================================================+
# || cat_status_line (CAT specific)                                       ||
# +========================================================================+
sub cat_status_line{
# [4] after webmin.pl
# This writes a Status bar
# hostname status customer date version
  my $hostname     = shift;
  my $remote_user  = shift;
  my $title_aref = shift;

  my $tcs = "<td NOWRAP align=center><font size='-2'>"; # table_column_start
  my $tce = "</font></td>";                             # table_column_end

  my $hostname_string  = $tcs."<b>$$title_aref[0]:</b> $hostname".$tce;# server
  my $date             =`date --iso-8601`;
  my $date_string      = $tcs."<b>$$title_aref[3]:</b> $date".$tce; # date
  my $status_string    = $tcs."<b>$$title_aref[1]:</b> $remote_user".$tce; # status
  my $version_string   = $tcs."<b>$$title_aref[4]:</b> ".&version($VERSION).$tce; # version
  my $customer_string  = $tcs.ucfirst($$title_aref[2]).$tce; # customer

  return "
  <table width='100%' align='center' cellpadding='0' cellspacing='0' border='0'>
    <tr bgcolor=#111111>
      <td>
        <table width='100%' align='center' cellpadding='0'  border='0'>
          <tr bgcolor=#eeeeee>
            $hostname_string
            $status_string
            $customer_string
            $date_string
            $version_string
          </tr>
        </table>
      </td>
    </tr>
  </table>
";
}
# +========================================================================+
# || cat_abc_nav (Some A-Z navigation bar)                                ||
# +========================================================================+
sub cat_abc_nav{
  my $script   = shift;
  my $abc_href = shift;
  my $s = "";
  foreach (('A'..'Z')){
    if($$abc_href{$_}){
      $s .= ' [<a href="'.$script.'?letter='.$_.'">'.$_.'</a>] ';
    }else{
      $s .= " [$_] ";
    }
  }
  $s .= "<br><br>\n";

  return $s;

}


# +========================================================================+
# || cat_explain_message                                                  ||
# +========================================================================+
sub cat_explain_message {
  my $message = shift;
  my $cb=shift;# webmin background VAR
  return "
<table width=50% cellpadding=5 cellspacing=5 border=0 $cb>
  <tr $cb><td><font color=$explain>$message</font></td></tr>
</table>";

}
# +========================================================================+
# || cat_error                                                            ||
# +========================================================================+
sub cat_error {
  my $m = shift;
  my $s = "<hr noshade><img src='images/c.png'> <table bgcolor=$red border=1><tr><td>$m";
  $s .= "</td></tr></table><hr noshade>";
  return $s;
}

# +========================================================================+
# || cat_footer (Webmin footer)                                           ||
# +========================================================================+
sub cat_footer{
  my $remote_user = shift;
  local *footer = shift; # webmin function with the same name
  my $support = shift; # support string from LDAP
  print "<hr>\n";
  if($remote_user eq "root" or $remote_user eq "admin" or $remote_user eq "cipadmin"){
    &footer("/cat","CAT");
    print $support;
#    print "<b>SUPPORT:</b> 1st Level:
#                           2nd Level: Martin Herweg:
#                           3rd Level: Christian Kuelker christian.kuelker\@cipworx.org";
  }else{
    &footer();
  }
}
# +========================================================================+
# || cat_print_html_error (CAT specific)                                  ||
# +========================================================================+
sub cat_print_html_error{
  my $error_code=shift;
  my $text_line=shift;
  print "<hr noshade>";
  print "<b>ERROR:</b> [$error_code]: ";
  print "<font color='$warning'>";
  print "$text_line";
  print "</font>";
  print "<br>";
  print "<hr noshade>";
}

# +========================================================================+
# || cat_table_header (HTML)                                              ||
# +========================================================================+
sub cat_table_header{
   my $column = shift;# number of column
   my $color = shift;
   my $title = shift;
   return "<table border=1><tr $color><th colspan='$column'>$title</th></tr>\n";
 }
# +========================================================================+
# || cat_table_row    (HTML)                                              ||
# +========================================================================+
 sub cat_table_row{
   my $h_aref = shift; # haed = boolen:  1=th, 2=td
   my $c_aref = shift; # colors of column (also number of columns)
   my $cont_aref = shift; # column content
   my $z=0;
   my $out = "  <tr>\n";
     foreach my $c (@$cont_aref){
       if($$h_aref[$z]){
	 $out .= "    <th $$c_aref[$z]>$c</th>\n";
       }else{
	 $out .= "    <td $$c_aref[$z]>$c</td>\n";
       }
       $z++;
   }
   $out .= "  </tr>\n";
   return $out;
 }

# +========================================================================+
# || cat_table_footer (HTML)                                              ||
# +========================================================================+
sub cat_table_footer{
  return "</table\n>";
}

sub cat_cipux_variable_col{
  my $script=shift;
  my $name=shift;
  my $value= shift;
  my $default = shift;
  my $system = shift;
  my $s = "<form method=post action='$script'>\n";
  $s .= "<input type=text value='$default' name='cipux_variable_value' maxlenght=255 size=50>\n";
  $s .= "<input type=submit value='setze' name='change_cipux_variable'>\n";
  $s .= "<input type=hidden name='$name' value='$value' >\n";
  $s .= "<input type=hidden name='system' value='$system' >\n";
  $s .= "</form>";
  return $s;
}

sub cat_passwd_col{
  my $script=shift;
  my $name=shift;
  my $value= shift;
#  my $s = "<form method=post action='$script'>\n";
  my $s .= "<input type=text name='change_pwd' maxlenght=12 size=12>\n";
  $s .= "<input type=submit value='setze' name='submit_password'>\n";
  $s .= "<input type=hidden name='$name' value='$value' >\n";
#  $s .= "</form>";
  return $s;
}

sub cat_open_passwd_col{
  my $script=shift;
  my $name=shift;
  my $value= shift;

  my $s .= "<input type=text value='$value' name='pwd' maxlenght=24 size=8>\n";
  $s .= "<input type=submit value='setze' name='submit_password'>\n";
  $s .= "<input type=hidden name='$name' value='$value' >\n";

  return $s;
}
sub cat_del_col{
  my $script=shift;
  my $name=shift;
  my $value= shift;
  my $hname=shift;
  my $hvalue= shift;
  my $s = "<input type=submit value='$value' name='$name'>\n";
  $s .= "<input type=hidden name='$hname' value='$hvalue' >\n";
  return $s;
}

sub cat_quota_usage_cols{
  #($bc,$bs,$bh,$bt,$ic,$is,$ih,$it);
  #my $SCRIPT = caller();
  my ($bc,$bs,$bh,$bt, $ic,$is,$ih,$it) = @_;
  #&log($SCRIPT,"&cat_quota_usage_cols($bc,$bs,$bh,$bt,$ic,$is,$ih,$it)");
  my $string = "<td>$bc</td>";
  if( $bs == 0 and $bh == 0){# green, no quota
    $string = "&nbsp;";
  }elsif($bc > $bh){    # red
    $string = "<p align=right><font color=$red>$bc</font> <img src='images/c.png' alt='EXEED'></p>";
  }elsif($bc > $bs){# orange
    $string = "<p align=right><font color=$orange>$bc</font><img src='images/w.png' alt='WARN'></p>";
  }else{
    # green
    $string = "<p align=right><b>$bc</b> <img src='images/r.png' alt='OK'></p>";
  }
  #  $string .="[$bc,$bs,$bh,$bt, $ic,$is,$ih,$it]";
  return $string;
}

sub cat_quota_set_col{
  my $script = shift; # caller CGI
  my $name   = shift; # some hidden name: user
  my $value  = shift; # some hidden value: bbeutlin
  my $quota  = shift; # Block hard quota
  my $s .= "<input type=text name='size' maxlenght=24 size=9 value='$quota'>\n";
  $s .= "<input type=submit value='setze' name='action'>\n";
  $s .= "<input type=hidden name='$name' value='$value' >\n";
  return $s;
}

sub cat_button_col{
  my $script  = shift;
  my $h_name  = shift;  # hidden        "group"
  my $h_value = shift;  # hidden        "class_of_84"
  my $a_name  = shift;  # action       "delete" -> parsed by script: submit_delete
  my $a_value = shift;  # translation: "delete Group"
 # my $s = "<form method=post action='$script'>\n";
  my $s .= "<input type=submit value='$a_value' name='submit_$a_name'>\n";
  $s .= "<input type=hidden name='$h_name'  value='$h_value' >\n";
 # $s .= "</form>";
  return $s;
}

sub cat_choose_column_col{
  my $n = shift; # number
  my %v = (
	   cipuxFirstname   => '*- Vorname',
	   cipuxLastname    => '*- Nachname',
	   course           => '-- Gruppe',
	   cipuxRole        => '-- Rolle',
	   password         => '-- Passwort (Klartext)',
	   uid              => "-# Kennung (Login)",
	   cipuxUserQuota   => '-- Nutzer Quota',
	   cipuxGroupQuota  => '-- Gruppen Quota'
	  );
#	   cipuxSoftQuota   => '- Minmale Obergrenze der Plattennutzung in KB',
#	   cipuxHardQuota   => '- Maximale Obergrenze der Plattennutzung in KB'
  my $s = "<select name='head$n'>\n";
  $s .= "  <option selected value='ignored'>Spalte ignorieren</option>\n";
  foreach my $c (sort keys %v){
    $s .= "  <option value='$c'>$v{$c}</option>\n";
  }
  $s .= "</select>\n";
}

sub cat_file_upload{

}


sub cat_action_status{
  my $status_aref = shift;

  my $s = "<hr noshade>";
  $s .= "<ul>\n";
  foreach my $line (@$status_aref){
    $s .= "  <li>$line</li>\n";
  }
  $s .= "</ul>\n";
  $s .= "<hr noshade>\n";
  return $s;

}
sub cat_warn_dialog{
  my $script = shift;
  my $title =shift;
  my $message=shift;
  my $user=shift;
  my $role =shift;
  my $returnname= shift;
  my $chancelvalue=shift;
  my $letter      =shift;
  print "<br><table align=center width=60%  cellpadding='5' cellspacing='5' border=1 bgcolor='#994444'><tr><th>$title</th></tr><tr><td>";

  print "$message";
  print "</td></tr><tr><td align=center>";
  print "<form method=post action='$script'>";
  print "<input type=hidden name=user value=\"$user\" >";
  print "<input type=hidden name=role value=\"$role\" >";
  print "<input type=hidden name=letter value=\"$letter\" >";
  print "<input type=submit name=$returnname value='OK'> ";
  print "<input type=submit name=action value=\"$chancelvalue\"> ";
  print "</form>";
  print "</td></tr></table><br>";
}
sub cat_error_info_dialog{
  my $script = shift;
  my $title =shift;
  my $message=shift;
  my $user=shift;
  my $role =shift;
  my $returnname= shift;
  my $letter = shift;

  print "<br><table align=center width=60%  cellpadding='5' cellspacing='5' border=1 bgcolor='#994444'><tr><th>$title</th></tr><tr><td>";
  print "$message";
  print "</td></tr><tr><td align=center>";
  print "<form method=post action='$script'>";
  print "<input type=hidden name=user value=\"$user\" >";
  print "<input type=hidden name=role value=\"$role\" >";
  print "<input type=hidden name=letter value=\"$letter\" >";
  print "<input type=submit name=$returnname value='OK'> ";
  print "</form>";
  print "</td></tr></table><br>";
}


sub cat_confirm_dialog{
  my $script = shift;
  my $title =shift;
  my $message=shift;
  my $user=shift;
  my $role =shift;
  my $returnname= shift;
  my $chancelvalue=shift;
  my $letter      =shift;
  print "<form method=post action='$script'>";
  print "<br><table align=center width=60%  cellpadding='5' cellspacing='5' border=1 bgcolor='#66cc66'><tr><th>$title</th></tr><tr><td>";

  print "$message";
  print "</td></tr><tr><td align=center>";

  print "<input type=hidden name=user value=\"$user\" >";
  print "<input type=hidden name=role value=\"$role\" >";
  print "<input type=hidden name=letter value=\"$letter\" >";
  print "<input type=submit name=$returnname value='OK'> ";
  print "<input type=submit name=action value=\"$chancelvalue\"> ";
  print "</td></tr></table><br>";
  print "</form>";
}

sub version{
  my $v = shift;
  #$v = 3.001000
  my ($major,$minor) = split(/\./,"$v");
  my $snd = substr($minor,0,3);
  my $srd = substr($minor,3,3);
  my $v1 = int($major);
  my $v2 = int($snd);
  my $v3 = int($srd);

  return "$v1.$v2.$v3";
}

1;

__END__

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>

See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut