#!/usr/bin/perl -w
# *========================================================================*
# || cipux_add
# || This script adds nodes to the CipUX system: Nodes may be users
# || and other. To fullfill this task, it adds the node to the CipUX
# || LDAP Server and may on behalf execute some shell scripts.
# || The Benefit of using cipux_add is a variable LDAP schema for different
# || Servers which may be edited by non LDAP experts. It also has some
# || abilities to auto calculate some to be filled in LDAP values.
# ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!
# || Licence:   GPLv2 (GPL version 2 or later)
# || 
# || Revision:  2005-08-02
# ||
# *========================================================================*


# TODO:
# [ ]  write a genaral function like $function, which checks the input data:
# (x) conversion_name conversion_login, ...
# (2) check cipuxClientNumber if range match
# (7) cipuxImageNumber for range 001..999


# check options
# open templatefile
# compute LDIF:
# * replace every line with --attribute command line option (MANDATORY, AUTO and other)
# * search for AUTO, and replace with AUTO generated values
# * are there overseen MADATORY or AUTO?


=pod

=head1 NAME

cipux_add - Version <[CIBOT::VERSION]>  2005-01-02 CipUX

=head1 SYNOPSIS

	cipux_add [OPTIONS]

	Options: (one nessessary)
        (01)  -u            :adds a user node, needs -w
        (02)  -c            :adds a course node
        (03)  -m            :adds a machine node
        (04)  -r            :adds a room node
        (05)  -t            :adds a hardware type node
        (06)  -i            :adds a image node
        (07)  -s            :adds a image slot node
        (08)  -y            :adds a admin node
        (09)  -x            :adds a configuration node
        Other options:
        (10)  -w <password> :password for a new node
                            (only if -u -c or -m option was given)
        (11)  -h            :get help message

        Attribute options:
             like:
             --attribute option=value
             example:
             --attribute cipuxFirstname=Christian

        See cipux_add -h for more attribute options!

        Return values:

        If succsess, returns login

        If error, returns LDAP and CipUX error message

=head1 ABSTRACT

This script adds nodes to the CipUX system: Nodes may be users
and other. To fullfill this task, it adds the node to the CipUX
LDAP Server and may on behalf execute some shell scripts.

The Benefit of using cipux_add is a variable LDAP schema for different
Servers which may be edited by non LDAP experts. It also has some
abilities to auto calculate some to be filled in LDAP values.

This script cipux_adds a CipUX node (LDAP entry) to the server.
Possible nodes are:

  User (-u),
  Course (-c),
  Machine (-m),
  Room (-r),
  Hardware Type (-t),
  Image (-i),
  Image Slot (-s),
  Admin (-y) or
  Configuration (-x)

It uses the ldapmodify programm and *.tpl files to perform the task.

=head1 REQUIREMENTS

       cipux::cibot
       cipux::conversion
       cipux_get_value
       cipux_list
       cipux_setup
       user.tpl
       course.tpl
       machine.tpl
       room.tpl
       hardware_type.tl
       configuration.tpl
       image.tpl
       image_slot.tpl
       admin.tpl
       ldapmodify
       Tie::IxHash
       Getopts::Long

=cut

package cipux_add;

# +========================================================================+
# |                           Global Part                                  |
# +========================================================================+

my $BRAND    = "cipworx";
my $PROJECT  = "cipux";
my $SCRIPT   = "cipux_add";
my $VERSION  = "<[CIBOT::VERSION]>";
my $BUILD    = "<[CIBOT::BUILD]>";

# global variables
my %option = ();            # the command line option relation
my $mandatory_options = ''; # lists needed options for usgae message
my %attribute = ();         # the given LONG command line option values
my %opt = ();               # the given SHORT command line option values
my $option = '';            # remember the given SHORT command line option type: -u
my $type_lc = '';           # remember the resulting node typ in lower case: user
my @modify = ();            # the changed version of TPL file
my $node_password = '';   # -w
my $replace = '';           # string to be echoed to ldapmodify
my $node = '';            # node to be added: chkuelker
my $node_type   = '';     # remember this: User,Course, ...

&log($SCRIPT,"="x76);
#&log($SCRIPT,"- Customer [$cipux{Customer}]");
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# +========================================================================+
# |                           Module                                       |
# +========================================================================+

# Load all modules
use strict;
use CipUX::CiBot;
use CipUX::Conversion;
use Tie::IxHash;
use Getopt::Long;

# ==========================================================================
# ||                       Node Structure                               ||
# ==========================================================================

# u c m r t i s y x
%option = ( U => 'User',
	    C => 'Course',
	    M => 'Machine',
	    R => 'Room',
	    T => 'Hardware_Type',
	    I => 'Image',
	    S => 'Image_Slot',
	    Y => 'Admin',
	    X => 'Configuration',
	  );

# ==========================================================================
# ||                       environment                                    ||
# ==========================================================================

=pod

=head2 environment

uses Getopts::Long to define additional LDAP attributes as a commandline
option.

It also defines the long options as the following:

  my %attribute = ();
  GetOptions("attribute=s" => \%attribute);

When used with command line options:

  --attribute uid=testinus --attribute homeDrive='/home/testinus'

this will generate the hash:

  $attribute{uid}='testinus';
  $attribute{homeDrive}='/home/testinus';

=cut

GetOptions(
	   "w=s" => \$opt{'w'},
	   "u"   => \$opt{'u'},
	   "c"   => \$opt{'c'},
	   "m"   => \$opt{'m'},
	   "r"   => \$opt{'r'},
	   "t"   => \$opt{'t'},
	   "i"   => \$opt{'i'},
	   "s"   => \$opt{'s'},
	   "y"   => \$opt{'y'},
	   "x"   => \$opt{'x'},
	   "h"   => \$opt{'h'},
	   "attribute=s" => \%attribute
	  );

# remember the given type: -u -c, -m, ...
foreach (keys %option){
  $mandatory_options .= " -".lc($_);
  my $o = lc($_);
  if(defined($opt{$o})){
    $option = $o;                             # ??
    $node_type   = $option{$_};             # remember this: User,Course, ...
    $type_lc = lc($option{$_});               # u
    &log($SCRIPT,"- got option [$o] for type [$option{$_}]");
  }
}

# if it is help, we can exit here:
if(defined($opt{h})){usage();exit(0)}

# no mandatory option, complain
if($option eq ''){
  &usage;&error('0100')
}

# comment this out, if you need the options for debug
# foreach my $o (keys %attribute){
#   &log($SCRIPT,"- got option --attribute [$o]=[$attribute{$o}]");
# }

# We need a password for specific options 
if( defined($opt{'u'}) or defined($opt{'c'}) or defined($opt{'y'}) ){
  if(!defined($opt{w})){
    usage();
    &error('0106');# cipux_add: The script needs the -w option and a password
    exit(1)
  }elsif($opt{w} eq ''){
    $node_password = &cibot_random_password();
    $opt{w} = $node_password;
    &log($SCRIPT,"The password is empty and will be set to the random value: [$node_password]");
    #usage();
    # cipux_add: Empty option -w, please specify password
    #  &error('0136');
    #   exit(1)
  }else{
    $node_password=$opt{w};
    &log($SCRIPT,"- got password: [$node_password]");
    ($attribute{userPassword},
     $attribute{sambaLMPassword},
     $attribute{sambaNTPassword}) = &cibot_get_password($node_password);
    $attribute{userPassword} = "{crypt}$attribute{userPassword}";# TODO: make this flexible
    &log($SCRIPT,"- user [$attribute{userPassword}] sambaNT [$attribute{sambaNTPassword}] sambaLM [$attribute{sambaLMPassword}]");
  }
}

# ==========================================================================
# ||                       check if node exsist                         ||
# ==========================================================================

# check for existens of the node:
# Example: 
#   $option = u
#   $search = Ldap_DN_U_Search
#   $cipux{$search} = uid
#   $attribute{$cipux{$search}} = bilbob  came from --attribute uid=bilbob
#
my $search = "Ldap_DN_".uc($option)."_Search";

# this searches for U: bibeutlin bibeutlin
#                   C: class84 class84
# my $node_href = &cibot_list(uc($option),$cipux{$search});

my $attribute = '';
my $node_href =  "";

if($option eq 'u'){
  $attribute = 'gidNumber';
  $node_href =  &cibot_list('C',$attribute);
}elsif($option eq 'c'){
  $attribute = 'gidNumber';
  $node_href =  &cibot_list('C',$attribute);
}elsif($option eq 'm'){
  $attribute = 'uidNumber';
  $node_href =  &cibot_list('M',$attribute);
}else{
  $attribute = cipux{$search};
  $node_href =  &cibot_list('C',$attribute);
}

# NEXT FREE ID-NUMBER (uidNumber,gidNumber)
my $id = 0;
if($option eq 'c' or $option eq 'm'){# no calc for uidNumber for USER, because it is eq gidNumber
  $id = &cibot_get_next_idNumber();
  $attribute{uidNumber}=$id;
  if($option eq 'c'){
    $attribute{gidNumber} = $id;
  }
  &log($SCRIPT,"- id = [$id]");
}


#print "attribute $attribute\n";


#if($option eq 'c'){# search only for COURSE
#  # calculate next free idNumber
#  # (1) reverse the hash
#  my %idNumber = ();
#  foreach my $key(%$node_href){
#    if(defined($$node_href{$key})){
#      if($$node_href{$key} ne ''){
#	$idNumber{$$node_href{$key}}=1;
#	#&log($SCRIPT,"- idNumber [$$node_href{$key}]");
#      }
#    }
#  }
#  # (2)
#  
#  foreach my $i ($cipux{Pam_UID_Start} .. $cipux{Pam_UID_End}) {
#    &log($SCRIPT,"- i [$i]");
#    if(not defined($idNumber{$i})){
##      #my $hex= _uid= sprintf("%x", $i);
#      $attribute{gidNumber} = $i;
#      $id = $i;
#      last;
#    }
#  }
#}elsif($option eq 'u'){# is a USER?
if($option eq 'u'){# test if GROUP exist for USER
  if(defined($$node_href{$attribute{uid}})){
    #print "UID $attribute{uid}";
    $attribute{uidNumber}=$$node_href{$attribute{uid}};
    $id = $$node_href{$attribute{uid}};
  }else{
    # error::0198=cipux_add: There is no group for this uid (user login), it should not be created:
    &error('0198',$attribute{uid});
    exit(1);
  }
}else{# do we nee a idNumber?

}

if($id >=$cipux{Pam_UID_End}){
  &error('0114');# ERROR: No UID left on device CipUX LDAP
  exit(0);
}
#
#
if($option ne 'u'){
  if(defined($attribute{$cipux{$search}})){
    if(defined($$node_href{$attribute{$cipux{$search}}}) ){
      &log($SCRIPT," * ERROR <0189>: The node you would like to add already exist: [$$node_href{$attribute{$cipux{$search}}}]");
      &error('0189',"id: $cipux{$search} = $attribute{$cipux{$search}}, idNumber = $$node_href{$attribute{$cipux{$search}}}" ); #error:0189=The node you would like to add already exist:
    }else{
      $node = $attribute{$cipux{$search}};
    }
  }
}

$node = $attribute{$cipux{$search}};


# ==========================================================================
# ||                       open template file                             ||
# ==========================================================================
# for 'User': $template_file = "/etc/cipux/default/user.tpl";
#my $template_file = "/etc/cipux/$cipux{Customer}/".$type_lc.".tpl";
my $template_file = "/etc/cipux/ldap/".$type_lc.".tpl";

&log($SCRIPT,"- open template file [$template_file]");

my $pid=open(FILE,$template_file)
  or &error('0102',$template_file); # cipux_add: Can't open template file for reading:
my @template_file=<FILE>;	
my $ok=close(FILE);
error('0103') if !defined($pid); # cipux_add: Error occured while open template file (no PID)
error('0104') if !$ok; # cipux_add: Error occured while closing template file

# ==========================================================================
# ||                       parse template file                            ||
# ==========================================================================

# LDIF example
#cipux_add: - MODIFY: dn: cn=LOGIN,ou=Group,dc=skole,dc=skolelinux,dc=no
#objectclass: posixGroup
#objectclass: top
#objectclass: lisGroup
#objectclass: cipuxGroup
#cn: LOGIN
#gidNumber: 10172
#groupType: private
#memberUid: LOGIN
#userPassword: {crypt}5a5yPRx/8w3k
#description: LOGIN
#cipuxSoftQuota: 20000
#cipuxHardQuota: 50000
#cipuxCreationDate: 2005-03-13
#cipuxRemark: CAT created

foreach my $line (@template_file){#                 for ever line of the template
  chomp($line);
  &log($SCRIPT,"- TPL line [$line]");
  if($line =~ m/^#/ or $line eq ''){ #               is this a line to skip?
    &log($SCRIPT,"- not using this line");
  }elsif($line =~ m/\:/){#                           is this a line not to skip?
    my ($attribute, $value) = split(':',$line,2);
    # delete trailing and leading white spaces:
    $attribute =~ s/^\s*|\s*$//g;
    $value     =~ s/^\s*|\s*$//g;
    &log($SCRIPT,"- attribute [$attribute] value [$value]");
    ####
    if(lc($attribute) eq 'objectclass'){#                    is this an node class?
      &log($SCRIPT,"- [$attribute] objectclass, skip");
      #      push(@modify,"$attribute: $value\n"); # store objectclass 
    }else{#                                                  no it is not an node class
      &log($SCRIPT,"- [$attribute] no objectclass, check for replacement");
      $value = $attribute{$attribute} if defined($attribute{$attribute});
      # if the attribute of the line is NOT the <MANDATORY=attribute> called 'secondary MANDATORY'
      if($value =~ m/MANDATORY/){#                           are there on ore more MANDATORY?
	&log($SCRIPT,"- may this be a seconday MANDATORY [$value]?");
	# /BEGIN((?:(?!BEGIN).)*)END/
	while($value =~ m/.*?<MANDATORY=((?:(?!>).)*)>/g){
	  my $match=$1;
	  &log($SCRIPT,"- match test for MANDATORY [$1]=[$match]?");
	  if(defined($attribute{$match})){
	    &log($SCRIPT,"- replacement [$match] with [$attribute{$match}]");
	    $value =~ s/<MANDATORY=$match>/$attribute{$match}/ge;
	  }else{
	    &log($SCRIPT,"* ERROR: The required attribute was not given: $match");
	    # cipux_add: The required attribute was not given (--attribute ATTRIBUTE=VALUE):
	    &error('0107',$match);
	  }
	}
      }
      if($value =~ m/AUTO/){
	&log($SCRIPT,"- AUTO attribute found");
	# /BEGIN((?:(?!BEGIN).)*)END/
	while($value =~ m/.*?<AUTO=((?:(?!>).)*)>/g){
	  my $match=$1;
	  &log($SCRIPT,"- match test for AUTO [$1]=[$match]?");
	  my $function = '';
	  my $v = '';
	  if(defined($attribute{$match})){# use an old value if exists
	    $v = $attribute{$match};
	    &log($SCRIPT,"- replacement (old)  [$match] with [$v]");
	  }else{# we have to compute a new value ...
	    $function = "&".$match.'($option)';
	    $v = '';
	    eval { $v = eval($function) };
	    if ($@) {
	      &log($SCRIPT,"* WARNING: the function [$function] raised an exeption: [$@]");
	    }
	    &log($SCRIPT,"- got value [$v] from function [$function]")if defined($v);
	    &log($SCRIPT,"- got value [EMPTY] from function [$function]")if not defined($v);
	  }
	  &log($SCRIPT,"- replacement (new generated)  [$match] with [$v]") if defined($v);
	  &log($SCRIPT,"- replacement (new generated)  [$match] with [EMPTY]")if not defined($v);
	  if($v eq ''){
	    # error::0197=cipux_add: Got an empty return value for an AUTO function, 3rd Level Support needed?
	    &error('0197',$function);
	  }else{
	    $value =~ s/<AUTO=$match>/$v/ge;
	    $attribute{$match}=$v; # to calculate AUTO only once
	    &log($SCRIPT,"- remember attribute [$match] value [$v]");
	  }
	}
	#;
      }
    }
    ####
    push(@modify,"$attribute: $value\n");
  }else{ # brocken TPL file
    foreach (@modify){
      chomp($_);
      &log($SCRIPT,"- LDIF: [$_]");
    }
    # error::0195=cipux_add: The template file is broken:
    &error('0195',$template_file);
  }
}

# ==========================================================================
# ||                       compute LDIF                                   ||
# ==========================================================================
foreach (@modify){
  chomp($_);
  &log($SCRIPT,"- LDIF: [$_]");
  $replace .= "$_\n";
}
# ==========================================================================
# ||                       execute ldap command                           ||
# ==========================================================================
my $cmd = "echo -en \"$replace\" | $cipux{Ldap_Modify_Command} -a -x -r -c -w $cipux{Ldap_Password} -h $cipux{Ldap_Host} -p $cipux{Ldap_Port} $cipux{Ldap_Start_TLS}  -D '$cipux{Ldap_Root_DN}'";

&log($SCRIPT,"- cmd [$cmd]");
&log($SCRIPT,"- replace [$replace]");

# do the bad thing, modiify or add the LDAP!!!
#my $pid = open(MODIFY,"$cmd > /tmp/ldap.error 2>&1")
 $pid = open(MODIFY,"$cmd 2>&1 |") # FIXME BIDIRECTIONAL PIPE
  or &error('0109'); # cipux_add: Can't execute LDAP modyfiy (add) command, to add user
#rint MODIFY $replace;
&log($SCRIPT,"- MODIFY: $replace");
my @debug_output = <MODIFY>;
 $ok=close(MODIFY);

my $error_message = '';
foreach(@debug_output){
  chomp();
  &log($SCRIPT,"- debug: [$_]");
  my $error_message .= $_;
}
&error('0110') if !defined($pid); # cipux_add: Error occured while executing LDAP modify (no PID)
&error('0111',$error_message) if !$ok; # cipux_add: Error occured wile executing LDAP modify (no bind or empty search)


# add a samba part to the LDAP account
if($cipux{Cipux_Use_Samba} eq "yes"){
  &log($SCRIPT,"- Cipux_Use_Samba = yes");
  if($node_type eq "User"){
    &log($SCRIPT,"- node type eq User");
    my $cmd = "/usr/bin/smbpasswd";
    my $opt = "$attribute{uid} >>/var/log/cipux/cipux_add.log 2>&1";
 
    # error::0200=cipux_add: smbpasswd is not there:
    &error('0200',$cmd) if( ! -x $cmd);
  
    if($attribute{uid} ne ""){
      # add a user to samba
      &log($SCRIPT,"- cmd [$cmd -a -s $opt] [$node_password]");
      open(CMD,"| $cmd -a -s $opt");
      print CMD "$node_password\n$node_password\n";
      close(CMD);
    }else{
      &log($SCRIPT,"- WARNING: try to set smbpasswd root. This should NOT happen for a user!]");
    }
  }elsif($node_type eq "Machine"){
    &log($SCRIPT,"- node type eq Machine");
    my $cmd = "/usr/bin/smbpasswd";
    my $opt = "$attribute{uid} >>/var/log/cipux/cipux_add.log 2>&1";

    # error::0200=cipux_add: smbpasswd is not there:
    &error('0200',$cmd) if( ! -x $cmd);

    if($attribute{uid} ne ""){
      # add a machine to samba
      &log($SCRIPT,"- cmd [$cmd -m -s $opt] [$node_password]");
      open(CMD,"| $cmd -m -a $opt");
      print CMD "$node_password\n$node_password\n";
      close(CMD);
    }else{
       &log($SCRIPT,"- WARNING: try to set smbpasswd root. This should NOT happen for a machine!]");
    }

  }
}else{
  &log($SCRIPT,"- Cipux_Use_Samba = no");
}


# parse the add.conf and execute the commands:
#my $config_file   = "$Cipux_Etc_Dir/$cipux{Customer}/cibot/$Cibot_Del_Conf_File";
#my $config_file   = "/etc/cipux/$cipux{Customer}/add.conf";
my $config_file   = "/etc/cipux/cibot/add.conf";


my $file_array_ref = "";
if(defined($node_password)){
  $file_array_ref = &cibot_parse_command_file($node_type,$config_file,$node,$node_password);
}else{
  $file_array_ref = &cibot_parse_command_file($node_type,$config_file,$node);
}

foreach my $cmd (@$file_array_ref){
  &log($SCRIPT,"- add.conf: cmd [$cmd]");
  &cibot_exec_cmd(undef,$cmd,'');
}

exit(0);

# +========================================================================+
# |                  Subroutines                                           |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

  print "-"x76;
  print "\n";
  print "($BRAND-$PROJECT-$SCRIPT-$VERSION) \n\n";
  print "USAGE:\n";
  print "       $SCRIPT [OPTIONS]\n";
  print "Add a NEW node!\n";
  print "Options: (need only one)\n";
  print "(01)  -u            :adds a user\n";
  print "(02)  -c            :adds a course\n";
  print "(03)  -m            :adds a machine\n";
  print "(04)  -r            :adds a room\n";
  print "(05)  -t            :adds a hardware type\n";
  print "(06)  -i            :adds a image\n";
  print "(07)  -s            :adds a image slot\n";
  print "(08)  -y            :adds a admin\n";
  print "(09)  -x            :adds a configuration\n";
  print "Other Options:\n";
  print "(10)  -h            :this message\n";
  print "(11)  -w <password> :if new user,course,machine node\n";
  print "\n";
  print "Attribute options: (mostly optional, some mandatory)\n";
  #print "  If set, they will replace default values from template file:\n";
  #print "    user.tpl    for users\n";
  #print "    course.tpl  for courses\n";
  #print "    machine.tpl for machines\n";
  #print "    room.tpl    for rooms\n";
  # print "  - An attribute containing <AUTO=string> should not be given on \n";
  # print "    the command line, because it will be computed and autofilled in!\n";
  # print "  - If there is a <SOME_THING> field in the attribute list,\n";
  # print "    this attribute must be given. You should use an attribute, where\n";
  # print "    its the only value! If there are more then one <WHAT_SO_EVER>\n";
  # print "    of the same kind, only ONE have to be given!\n";
  print "  - An attribute with '[M]' MUST be given on commnadline!\n";
  # -u bis -r


#  foreach my $node_option (@option){
#    print"* ($option{$node_option}) node command line options for $node_option:\n";
#    my $file=$template{$node_option}; 
#    my %attribute = &template($file);

#    foreach my $option (keys %attribute){

      # if NOT dn, objectclass or <AUTO...>
#      if(not lc($option) =~m/^objectclass.*/ # cobjectclass03, ...
#	 and lc($option) ne "dn"
#	 and not $attribute{$option}=~m/.*<AUTO.*/
#    	){
#        my $string    = "--attribute $option=value";
#        my $mandatory = "";
#        my $default   = "";
#        if(defined($attribute{$option})){
#	  if($attribute{$option}=~m/.*<MANDATORY.*/){
#            $mandatory = "  [M] ";
#            $default   = "";
#	  }else{
#            $mandatory = "      ";
#	    $default   = ", default: $option=$attribute{$option}";
#	  }
#	 }
#	print "$mandatory$string$default\n";
#      }
#    }
#  }
  print "EXAMPLE:\n";
  print "[1] To add a NEW room to $PROJECT use the following command:\n";
  print "      cipux_add -r -p secret --attribute cn=myroom\n";
  print "    This will create a new room with the name 'myroom'\n";
  print "    and the other attributes will be default values.\n";
  print "    As you can see above (under -r).\n";
  print "[2] To add a NEW user: )\n";
  print "      cipux_add -u -p ldap_secret -w new_user_secret \n";
  print "                          --attribute cipuxFirstname=Christian \n";
  print "                          --attribute cipuxLastname=Kuelker\n";
  print "[3] To add a NEW user: (with some strange values!)\n";
  print "      cipux_add -u -p ldap_secret -w new_user_secret \n";
  print "                          --attribute cipuxFirstname=Christian \n";
  print "                          --attribute cipuxLastname=Kuelker\n";
  print "                          --attribute homeDirectory='/tmp/ckuelker'\n";
  print "    Will set the homedir into the tmp directory. If you skip\n";
  print "    the first two (First-, Lastname)', you will get an error!\n";
  print "    You should at least have one cn or equal entry, because one can\n";
  print "    not determine the username/login out of a given directory.\n";
  print "    For a user the cn entry will auto constructed from the first and\n";
  print "    lastname.\n";
  print "[4] Hint:\n";
  print "      --attribute uidNumber=1234\n";
  print "      --attribute gidNumber=1234\n";
  print "    The two above are the same! (but have no effect on command line)\n";
  print "    Because there are linked by <AUTO=uidNumber> in the template file!\n";
  print "    Don't change that unless, you're know what you're doing!!\n";
  print "[5]  --attribute 'gecos=My home = my castle' Is not a valid attribute.\n";
  print "    Because of the 2 equal signs!\n";
  print "-"x76;
  print "\n";
}

sub AUTOLOAD {
    use vars qw($AUTOLOAD);
    my $error = $AUTOLOAD;
    $error =~ s/.*:://;
    # error::0196=cipux_add: The AUTO replacement LDAP attribute is not defined as function in cipux_add! You have the choice: (1) fix the *.tpl file not to use this <AUTO=attribute> or (2) define a function in cipux_add with the name:
    &error('0196',$error);
}

sub uidNumber{
#  my $option = shift;
  &log($SCRIPT,"& uidNumber($option)");
#  my $idNumber= &cibot_get_next_idNumber($option,'uidNumber');
#  &log($SCRIPT,"-> return next idNumber [$idNumber]");
#  return $idNumber
}

sub gidNumber{
#  my $option = shift;
  &log($SCRIPT,"& gidNumber($option)");
#  my $idNumber = &cibot_get_next_idNumber($option,'gidNumber');
#  &log($SCRIPT,"-> retrun next idNumber [$idNumber]");
#  return $idNumber;
}

sub userPassword{
  # nothing to do, but must be defined
  # the work will be done, while checking for the -w switch
  return $node_password; # TODO: do not retrun a global
}

sub sambaNTPassword{
  # nothing to do, but must be defined
  # the work will be done, while checking for the -w switch
  return $attribute{sambaNTPassword};
}

sub sambaLMPassword{
  # nothing to do, but must be defined
  # the work will be done, while checking for the -w switch
  return $attribute{sambaLMPassword};

}

sub cipuxCreationDate{
  return &cibot_actual_date();
}


#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut


