#!/usr/bin/perl -w
# +========================================================================+
# || cipux_bot_extinct_from_courses                                       ||
# ||                                                                      ||
# || A robot, who extinct non existend logins from groups                 ||
# ||                                                                      ||
# || (c) Copyright 2005 by Christian Kuelker                              ||
# || License: GPL Version 2 or later                                      ||
# ||                                                                      ||
# || Revision 2005-08-05                                                    ||
# ||                                                                      ||
# +========================================================================+

=pod

=head1 NAME

cipux_bot_extinct_from_courses - Version <[CIBOT::VERSION]>  <[CIBOT::DATE]> CipUX

=head1 SYNOPSIS

 cipux_bot_extinct_from_courses -a   : deletes all not used member users

 cipux_bot_extinct_from_courses -h   : prints this help message

=head1 ABSTRACT

This script deletes a all nonexistent users uid  
as memberUid from the CipUX courses.

=head1 REQUIREMENTS

  CipUX::CiBot
  cipux_get_value
  cipux_set_valule

=head1 METHOD

 * search for all users (uid)
 * search for all members (memberUid) in courses (gid)
 * go for every member in every course
 * if he is not in the userlist, delete him from the course

=cut

package cipux_bot_extinct_from_courses;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_bot_extinct_from_courses";
my $VERSION = "<[CIBOT::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

# some globale VAR's:
my $run = 1;

# $fix_ldap = 1; this fix some problem of nonexistent members
# $fix_ldap = 0; do not add nonexistent members
my $fix_ldap = 1;

# $speedup = 1; uses /var/log/cipux/cipux_bot_extinct_from_courses.list
# $speedup = 0; uses /var/log/cipux/cipux_bot_extinct_from_courses.list
my $speedup  = 1;

# $verbose = 1; print whether to make a run or not, may be noisy fro crond
# $verbose = 1; not printing
my $verbose = 1;

&log($SCRIPT,"="x76);
&log($SCRIPT,"BEGIN LOG: $SCRIPT $VERSION $BUILD");
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# +========================================================================+
$PART=                       "Modules";#                                   |
# +========================================================================+
use strict;
use CipUX::CiBot;
use Getopt::Std;

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();
getopts("Ch",\%opt);

if(defined($opt{h})){usage();exit(0)}

if(defined($opt{C})){
  &log($SCRIPT,"- $SCRIPT got option -C");
}else{
  # cipux_bot_extinct_from_courses: 
  # The mandatory option -C was not given!
  &usage;&error('0199')
}

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

# * search for all users (uid)
my $uid_href = &cibot_list('U','uid');
my %uid_href_copy = %$uid_href;

if($speedup){
  $run = 0;
  if( -f '/var/log/cipux/cipux_bot_extinct_from_courses.list'){
    &log($SCRIPT,"- speedup file exists");
    my @uid = `cat /var/log/cipux/cipux_bot_extinct_from_courses.list`;
    foreach(@uid){
      chomp();
      &log($SCRIPT,"- speedup: exists? [$_]");
      if(not defined($$uid_href{$_})){
	$run = 1;
	last;
      }else{
	delete($uid_href_copy{$_});
      }
    }
  }else{# first run with speedup
    &log($SCRIPT,"- firstrun");
    $run = 1;
  }
}

if(not $run){
  # test the copy if something was left
  foreach (keys %uid_href_copy){
    &log($SCRIPT,"- changed data [$_]");
    $run = 1;
  }
}

if($run){
  &log($SCRIPT,"- do make a run");
  print "$SCRIPT: changes in LDAP: make a run\n" if $verbose;
  # * search for all members (memberUid) in courses (gid)
  my $courses_href = &cibot_list('C','memberUid',1);
  # * go for every member in every course
  foreach my $c (keys %$courses_href){
    &log($SCRIPT,"- parse course [$c]");
    foreach my $m (@{ $$courses_href{$c} }){
      &log($SCRIPT,"- check for member [$m]");
      if($m ne 'x'){
	if(not defined($$uid_href{$m})){
	  # * if he is not in the userlist, delete him from the course
	  &log($SCRIPT,"-> member user [$m] not existing!");
	  my $cmd = "/usr/bin/cipux_set_value "
	    ."-d -c '$c' -e memberUid -v '$m'";
	  &log($SCRIPT,"- delete member [$m] by cmd [$cmd]!");
	  &cibot_exec_cmd('term',$cmd,'');
	}
      }else{# $m == 'x'
	my $cmd = "/usr/bin/cipux_set_value "
	  ."-a -c '$c' -e memberUid -v '$c'";
	&log($SCRIPT,"WARNING: Course [$c] has no memberUid! It should at "
	     ."least [memberUid=$c]. ");
	if($fix_ldap){
	  &log($SCRIPT,"FIXING: Will fix this by cmd [$cmd]");
	  &cibot_exec_cmd('term',$cmd,'');
	}
      }
    }
  }
  # write
  if($speedup){
      `rm -f /var/log/cipux/cipux_bot_extinct_from_courses.list`;
    foreach (sort keys %$uid_href){
      &log($SCRIPT,"- write to cipux_bot_extinct_from_courses.list [$_]");
      `echo $_ >> /var/log/cipux/cipux_bot_extinct_from_courses.list`;
    }
  }
}else{
  &log($SCRIPT,"- no changes do not make a run");
  print "$SCRIPT: no changes in LDAP: do not make a run\n" if $verbose;
}

exit(0);

# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

  print "-"x76;
  print "\n";
  print "($BRAND-$PROJECT-$SCRIPT-$VERSION)\n\n";
  print "  Search for ALL not used member users (memberUid) and\n";
  print "  deletes the member users from all CipUX courses!\n\n";
  print "USAGE:\n";
  print "       $SCRIPT -C   : deletes all not used member users\n\n";
  print "       $SCRIPT -h   : prints this help message\n";
  print "\n";
  print "-"x76;
  print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
