#!/usr/bin/perl -w
# *========================================================================*
# || cipux_del                                                            ||
# || This script deletes a CipUX LDAP entry from the server. It uses then ||
# || the del.conf to delete the user from the system and to delete the    ||
# || home directory.                                                      ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2                                                     ||
# ||                                                                      ||
# || Revision:  2005-08-04                                                ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_del - Version <[CIBOT::VERSION]>  2005-01-02 CipUX

=head1 SYNOPSIS

	cipux_del [OPTIONS]

	Options: (one nessessary)
        (01)  -u <NAME>     :deletes a user node
        (02)  -c <NAME>     :deletes a course node
        (03)  -m <NAME>     :deletes a machine node
        (04)  -r <NAME>     :deletes a room node
        (05)  -t <NAME>     :deletes a hardware type node
        (06)  -i <NAME>     :deletes a image node
        (07)  -s <NAME>     :deletes a image slot node
        (08)  -y <NAME>     :deletes a admin node
        (09)  -x <NAME>     :deletes a configuration node
        Other options:
        (10)  -h            :get help message

=head1 ABSTRACT

This script deletes a CipUX node (LDAP entry) from the server.
Possible nodes are:

  User (-u),
  Course (-c),
  Machine (-m),
  Room (-r),
  Hardware Type (-t),
  Image (-i),
  Image Slot (-s),
  Admin (-y) or
  Configuration (-x)

=head1 REQUIREMENTS

  CipUX::CiBot
  del.conf
  ldapmodify
  Getopt::Std

=cut

package cipux_del;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_del";
my $VERSION = "<[CIBOT::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

my $mandatory_options = '';# this is for error message
my $node_option     = "";# -u -c -m -r
my $node_type       = "";# User, Course, Machine, Room
my $node            = "";# bebeutlin, gandalf, ...
my $c                 = 0;
my $dn                = "";
my $node_href       = "";

&log($SCRIPT,"="x76);
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# +========================================================================+
# |                           Module                                       |
# +========================================================================+

# Load all modules
use strict;
use CipUX::CiBot;
use Getopt::Std;

# u c m r t i s y x
my %option = ( U => 'User',
	       C => 'Course',
	       M => 'Machine',
	       R => 'Room',
	       T => 'Hardware_Type',
	       I => 'Image',
	       S => 'Image_Slot',
	       Y => 'Admin',
	       X => 'Configuration',
	     );
foreach (keys %option){
  $mandatory_options .= " -".lc($_);
}

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();

getopts("u:c:m:r:t:i:s:y:x:h",\%opt);

if(defined($opt{h})){usage();exit(0)}

foreach (keys %option){
  my $o = lc($_);
  if(defined($opt{$o})){
    $node        = $opt{$o};                # remember this: u,c, ...bibeutlin, gandalf, ...
    $node_option = $o;                      # remember this: u,c, ...
    $node_type   = $option{$_};             # remember this: User,Course, ...
    my $s1         = "Ldap_DN_".$_."_Prefix"; # example: Ldap_DN_U_Prefix
    my $s2         = "Ldap_DN_".$_;           # example: Ldap_DN_U_Prefix
    $dn            = $cipux{$s1}."=".$node.",".$cipux{$s2};
    $node_href = &cibot_list($_,$s1);
    $c++ ;
    &log($SCRIPT,"- got option [$o]");
    &log($SCRIPT,"- node [$node]");
    &log($SCRIPT,"- node_type [$node_type]");
    &log($SCRIPT,"- node_option [$node_option]");
    &log($SCRIPT,"- dn [$dn]");
  }
}

#error::0137=cipux_del: Need exactly one of the following options:
if($c==0){&usage;&error('0137',$mandatory_options)}

# ok
if($c==1){&log($SCRIPT,"- got exactly one option, good!")}

# +========================================================================+
$PART=                       "Check if Node exist";#                     |
# +========================================================================+
if(defined($$node_href{$node})){
  &log($SCRIPT,"- node exist, it meight be deleted");
}else{
  &log($SCRIPT,"- node is nonexistend, it meight not be deleted");
  &error('0150',$node);# error::0150=cipux_del: The node does not exist:
  exit(1);
}

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

# parse the del.conf and execute the commands:
#my $config_file   = "$Cipux_Etc_Dir/$cipux{Customer}/cibot/$Cibot_Del_Conf_File";
#my $config_file   = "/etc/cipux/$cipux{Customer}/del.conf";
my $config_file   = "/etc/cipux/cibot/del.conf";

my $file_array_ref = &cibot_parse_command_file($node_type,
					       $config_file,
					       $node);

#foreach my $cmd (@$file_array_ref){
#  print $cmd;
#  &log($SCRIPT,"- del.conf: cmd [$cmd]");
#  my $pid = open(CMD,"| $cmd")
#    # cipux_del: Can't execute del command:
#    or error('0141',$cmd);
#  my $ok=close(CMD);
#  # cipux_del: Error while executing del command:
#  &error('0142',$cmd) if !defined($pid);
#  # cipux_del: Executing del command not successful:
#  &error('0143',$cmd) if !$ok;
#}

foreach my $cmd (@$file_array_ref){
  &log($SCRIPT,"- del.conf: cmd [$cmd]");
  &cibot_exec_cmd(undef,$cmd,'');
}

# del a samba part to the LDAP account
if($cipux{Cipux_Use_Samba} eq "yes"){
  &log($SCRIPT,"- Cipux_Use_Samba = yes");
  if($node_type eq "User"){
    my $cmd = "/usr/bin/smbpasswd";
    my $opt = "$node >>/var/log/cipux/del.log 2>&1";
 
    # error::0201=cipux_del: smbpasswd is not there:
    &error('0201',$cmd) if( ! -x $cmd);
  
    # add a user to samba
    &log($SCRIPT,"- cmd [$cmd -d $opt]");
    open(CMD,"$cmd -d $opt");
    close(CMD);
  }
}else{
  &log($SCRIPT,"- Cipux_Use_Samba = no");
}

# /usr/local/bin/ldapdelete  -p 389 -h localhost -x -w SECRET
#                            -D 'uid=root,o=cipworx' d: ...

my $cmd = "$cipux{Ldap_Delete_Command} -D '$cipux{Ldap_Root_DN}' -p $cipux{Ldap_Port} -h $cipux{Ldap_Host} -x $cipux{Ldap_Start_TLS} -w $cipux{Ldap_Password} '$dn'";
#
&log($SCRIPT,"- cmd: $cmd");

my $pid = open(CMD,"$cmd|")
  # cipux_del: Can't execute ldap delete command:
  or error('0138',$cmd);
my $output = <CMD>;
my $ok = close(CMD);
# cipux_del: Error while executing ldap delete command:
 &error('0139',$cmd) if !defined($pid);
# cipux_del: Executing ldap delete command not successful:
 &error('0140',$cmd) if !$ok;
# TODO different exit values!
#      0 = ok
#      1 = LDAP not ok
#      2 = LDAP ok, but node not found 'No such node (32)'
#      3 = LDAP ok, but del.conf CMDs error



exit(0);

# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

  print "-"x76;
  print "\n";
  print "USAGE: ($BRAND-$PROJECT-$SCRIPT-$VERSION)\n";
  print "       $SCRIPT [OPTIONS]\n";
  print "Delete a node from CipUX!";
  print "\n";
  print "Options: (need only one)\n";
  print "(01)  -u <NAME>     :deletes a user\n";
  print "(02)  -c <NAME>     :deletes a course\n";
  print "(03)  -m <NAME>     :deletes a machine\n";
  print "(04)  -r <NAME>     :deletes a room\n";
  print "(05)  -t <NAME>     :deletes a hardware type\n";
  print "(06)  -i <NAME>     :deletes a image\n";
  print "(07)  -s <NAME>     :deletes a image slot\n";
  print "(08)  -y <NAME>     :deletes a admin\n";
  print "(09)  -x <NAME>     :deletes a configuration\n";
  print "Other Options:\n";
  print "(10)  -h            :this message\n";
  print "\n";
  print "-"x76;
  print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut

