#!/usr/bin/perl -w
# *========================================================================*
# || cipux_extinct_from_courses                                              ||
# ||                                                                      ||
# || This script deletes a given user from all CipUX courses (groups)     ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!||
# || Licence:   GPL version 2 or later                                    ||
# ||                                                                      ||
# || Revision:  2005-08-05                                                ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_extinct_from_courses - Version <[CIBOT::VERSION]>  2005-03-30 CipUX

=head1 SYNOPSIS

  cipux_extinct_from_courses -u <NAME>

  or

  cipux_extinct_from_courses -h
  get help message

=head1 ABSTRACT

This script deletes a given user from all CipUX courses (groups).
The script querries all groups for memberUid. If it matchs, it will
delete this memberUid from the courses.

=head1 REQUIREMENTS

  CipUX::CiBot
  cipux_get_value
  cipux_set_valule

=cut

package cipux_extinct_from_courses;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_extinct_from_courses";
my $VERSION = "<[CIBOT::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

# some globale VAR's:
my $member_attribute = "memberUid";
my $login = "";
my @cmd = ();

&log($SCRIPT,"="x76);
&log($SCRIPT,"BEGIN LOG: $SCRIPT $VERSION $BUILD");
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# +========================================================================+
$PART=                       "Modules";#                                   |
# +========================================================================+
use strict;
use CipUX::CiBot;
use Getopt::Std;

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();
getopts("u:h",\%opt);

if(defined($opt{h})){usage();exit(0)}

if(defined($opt{u})){
  $login = $opt{u};
  &log($SCRIPT,"- $SCRIPT got option -u with value [$login]");
}else{
  # cipux_extinct_from_courses: 
  # The mandatory option -u was not given! Please specify -u <LOGIN>!
  &usage;&error('0158')
}

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

my $get_value_cmd = "/usr/bin/cipux_get_value -C "
                    ."-e '$member_attribute'";

my $pid = open(CMD,"$get_value_cmd |")
# cipux_extinct_from_courses: Can't execute get_value command:
  or &error('0159',$get_value_cmd);
my @group_member = <CMD>;
my $ok = close(CMD);
# cipux_extinct_from_courses: Error while executing get_value command:
&error('0160',$get_value_cmd) if !defined($pid);
# cipux_extinct_from_courses: Executing get_value command not successful:
 &error('0161',$get_value_cmd) if !$ok;

# parse every group and make a command array
foreach my $group (@group_member){
  chomp($group);  
  &log($SCRIPT,"- parsing groups: [$group]");

  # group TAB member1 TAB member2
  my @multiple_values=(); #mul
  @multiple_values = split("\t",$group); #mul
  shift(@multiple_values);# the first one is the group not the member

  foreach my $member (@multiple_values){
    &log($SCRIPT,"- is member [$login] eq [$member]?");
    if($member eq $login){
      my $cmd = "/usr/bin/cipux_set_value "
	."-d -c '$multiple_values[0]' -e 'memberUid' -v '$member'";
      &log($SCRIPT,"-> yes! add cmd: [$cmd] to cheque");
      push(@cmd,$cmd);
    }
  }
}

# execute every 'memberUid delete' from cammand array
foreach my $cmd (@cmd){
  &log($SCRIPT,"- execute cmd [$cmd]");
  my $pid = open(CMD,"| $cmd")
    # cipux_extinct_from_courses: 
    # Can't execute del memberUid command from cammand array:
    or &error('0162',$cmd);
  my $ok=close(CMD);
  # cipux_extinct_from_courses: 
  # Error while executing del memberUid command from cammand array:
  &error('0163',$cmd) if !defined($pid);
  # cipux_extinct_from_courses: 
  # Executing del memberUid command not successful from cammand array:
  &error('0164',$cmd) if !$ok
}

exit(0);

# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

  print "-"x76;
  print "\n";
  print "($BRAND-$PROJECT-$SCRIPT-$VERSION)\n\n";
  print "  Search for a given member user (memberUid) and\n";
  print "  deletes the member user from all CipUX courses!\n\n";
  print "USAGE:\n";
  print "       $SCRIPT -u <NAME> : deletes a member user\n\n";
  print "       $SCRIPT -h        : prints this help message\n";
  print "\n";
  print "-"x76;
  print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
