#!/usr/bin/perl -w
# +========================================================================+
# || cipux_get_value                                                      ||
# || This script shows arbitrary LDAP values for CipUX.                   ||
# ||                                                                      ||
# ||                                                                      ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2                                                     ||
# ||                                                                      ||
# || Revision:  2005-08-02                                                ||
# ||                                                                      ||
# +========================================================================+

=pod

=head1 NAME

cipux_get_value - Version <[CIBOT::VERSION]>  2002-08-16 CipUX

=head1 SYNOPSIS

	cipux_get_value [OPTIONS] -e <ENTRY>

        Options: (exact only one is mandatory)
        (01) -u <UCN>  :get user value
        (02) -c <CCN>  :get course value
        (03) -m <MCN>  :get machine value
        (04) -r <RCN>  :get room value
        (05) -t <TCN>  :get hardware type value
        (06) -i <ICN>  :get image value
        (07) -s <SCN>  :get image slot value
        (08) -y <YCN>  :get admin value
        (09) -x <XCN>  :get configuration values
        (10) -U        :get value for all users
        (11) -C        :get value for all courses
        (12) -M        :get value for all machines
        (13) -R        :get value for all rooms
        (14) -T        :get value for all hardware types
        (15) -I        :get value for all images
        (16) -S        :get value for all image slots
        (17) -Y        :get value for all admins
        (18) -X        :get value for all configuration
        Auxilliary options:
        (19) -h        :get help page
        (20) -v        :get value for cipuxVariable

	UCN = U+CN = U=user + CN = canonical name
	CCN, MCN, RCN analog

=head1 ABSTRACT

This script querries a LDAP server for a given entry (attribute).
It can be done in two modes (A) all nodes and (B) one node to be
returned. Four fields can be querried: (1) user (2) course (3)
machine and (4) room.

The output of mode (A) will be a TAB seperated list: 'CN TAB ENTRY\n'.
The output of (B) a plain list or single entry.

This Version was made by www.cipworx.org for Skolelinux.

=head1 REQUIREMENTS

	CipUX:CiBot
        Getopt::Std
        Ldap_Search_Command

=head1 EXAMPLES

The command

  cipux_get_value -u cipadmin -e cipuxRole

will return: "account". Whereas

  cipux_get_value -c cipadmin -e cipuxRole

will return: "admin". And yhe line

 cipux_get_value -x cipux -e cipuxVariable -v Cipux_Default_Group_Hard_Quota

will return:

 0

if the attribute "cipuxVarible: Cipux_Default_Group_Hard_Quota=0" is set in LDAP.
But aware, there could be more than one return value, if the "cipuxVariable:
Cipux_Default_Group_Hard_Quota=..." is more then once in that ldap node "cipux".
This could be the case, if somebody add that value via some LDAP browser.

Whereas

 ./cipux_get_value -U -e uid

will retrun (for example):

 chkuelker
 geprade
 maklappenbach
 maherweg

as a list of all users of the system. But you should use cipux_task_list_users for this,
because it may not be the attribute uid on all systems and cipux_get_value will not
respect the cipuxRole, whereas cipux_task_list_users should.

=cut

package cipux_get_value;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_get_value";
my $VERSION = "<[CIBOT::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

# other global vars:
my %values=();
my @values=();
my %opt;
my $dn_search = "";
my $command = "";

&log($SCRIPT,"="x76);
&log($SCRIPT,"BEGIN LOG: $SCRIPT $VERSION");
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT $cml");

# Usage of cipworx cipux modules and libraries
use strict;
use CipUX::CiBot;
use Getopt::Std;
use MIME::Base64;

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment modules
my @one_mandatory_option=qw(u: c: m: r: t: i: s: y: x: U C M R T I S Y X);
my $options="v:e:h";
foreach(@one_mandatory_option){$options.=$_}
&log($SCRIPT,"- options: [$options]");

#getopts("e:hu:c:m:r:t:i:s:y:x:UCMRTISYX",\%opt);
getopts("$options",\%opt);

if(defined($opt{h})){&usage;exit(0);}

# (only if NOT root security)
#if(!defined($opt{p})){&usage;error('0039');}# cipux_get_value: No option -p, please give it to me
if(!defined($opt{e})){&usage;error('0040');}# cipux_get_value: No option -e, please give it to me

# remember choosen command line options:
my %test                      = ();
my $search_node             = "";
my $one_mandatory_option_list = "";
my $filter                    = "";# 'cn=*' only for -U -C -M ...
my $capital_option            = 0;
my $searchbase                = "";
my $dn_prefix                 = "";

foreach  (@one_mandatory_option){
  s/://g;# we don't need the : any more!
  if(defined($opt{$_})){
    # Example           x             cibot
    &log($SCRIPT,"- parse option '$_' with value '$opt{$_}'\n");
    $test{$_} = 1; # $test{U} = 1 if defined($opt{U});
    $search_node = "-$_"; # $search_node = "-U" if defined($opt{U});

    # searchbase
    # OLD: $searchbase=$Ldap_User_DN    if defined($opt{u}) or defined($opt{U});
    # NEW: $searchbase=$Ldap_DN_U
    chomp();
    my $string = "Ldap_DN_".uc($_); # example: Ldap_DN_U
    $searchbase = $cipux{$string};
    &log($SCRIPT,"- ldap searchbase string:");
    &log($SCRIPT,"  [$string] = [$searchbase]");
    # prefix
    # $dn_prefix="dn: ".$Ldap_DN_U_Prefix if defined($opt{U});
    $string .= "_Prefix"; # example: Ldap_DN_U_Prefix
    &log($SCRIPT,"- ldap dn prefix string [$string]");
    $dn_prefix = $cipux{$string};
    &log($SCRIPT,"  [$string] = [$dn_prefix]");
    $string =~ s/_Prefix/_Search/g; # example: Ldap_DN_U_Search
    &log($SCRIPT,"- ldap dn search string [$string]");
    $dn_search = $cipux{$string};
    &log($SCRIPT,"  [$string] = [$dn_search]");
    # 91 = Z+1, 64 = A-1   => cn=* if from A till Z
    if(ord($_) < 91 and ord($_) > 64){
      $filter=$dn_prefix.'=*' if ord($_) < 91 and ord($_) > 64; #TODO: replace uid with cn for real CipUX, uid is for Skole
     $capital_option = 1;
    }else{
      # $filter="cn=$opt{u}" if defined($opt{u});
      $filter="$dn_prefix=$opt{$_}"; #TODO: replace uid with cn for real CipUX, uid is for Skole
    }
  }
  $one_mandatory_option_list .= " -".$_;
}

  &log($SCRIPT,"- \$search_node  = '$search_node'");
  &log($SCRIPT,"- \$filter         = '$filter'");
  &log($SCRIPT,"- \$capital_option = '$capital_option'");
  &log($SCRIPT,"- \$searchbase     = '$searchbase'");
  &log($SCRIPT,"- \$dn_prefix      = '$dn_prefix'");
  &log($SCRIPT,"- \$dn_search      = '$dn_search'");

my $c=0;
foreach (sort keys %test){$c++}
# Need definitly one option from:
if($c==0){&usage;&error('0041',$one_mandatory_option_list)}
# ok
if($c==1){&log($SCRIPT,"- Got one mandatory option")}
# cipux_get_value: To many options, need only one auxiliary option
if($c>1){&usage;&error('0042',$one_mandatory_option_list)}

my $entry =$opt{e};

# +========================================================================+
$PART=              "Prepare search";#                                     |
# +========================================================================+

# /usr/local/bin/ldapsearch  -D 'cn=root,o=cipworx'
#                            -b 'ou=cipux,o=testux,o=cipworx'
#                            -w 12345678 -LLL uid=* gecos

# /usr/bin/ldapsearch -x -p 389 -h localhost -w GEHIEM -D 'cn =admin,ou=People,dc=skole,dc=skolelinux,dc=no' -b 'ou=People,dc=skole,dc=skolelinux,dc=no' -LLL cn=bilbob uid

if(defined($cipux{'Ldap_Password'})){
   &log($SCRIPT,"- Ldap_Password is NOT EMPTY ");
}else{
   &log($SCRIPT,"- WARNING: Ldap_Password is EMPTY ");
}

$command = "$cipux{'Ldap_Search_Command'} -x -p $cipux{'Ldap_Port'} -h $cipux{'Ldap_Host'}"
         ." -w '$cipux{'Ldap_Password'}' $cipux{Ldap_Start_TLS}  -D '$cipux{'Ldap_Root_DN'}'"
         ." -b '$searchbase'"
         ." -LLL $filter $entry";

&log($SCRIPT,"- CMD [$command]");

#if($DEBUG){
# foreach (@VAR){
#   s/'//g;# we don't need the ' any more!
#   &log($SCRIPT,"$_ = '".eval("$_")."'");
# }
#}

# +========================================================================+
$PART=              "Main part";#                                          |
# +========================================================================+

# do the command
my $pid=open(LDAP,"$command |")
  or &error('0009'); # cipux_get_value: Can't execute LDAP search command
my @output=<LDAP>;
my $ok=close(LDAP);
error('0013') if !defined($pid); # cipux_get_value: Error occured while executing LDAP search (no PID)
error('0059') if !$ok; # cipux_get_value: Error occured while executing LDAP search (no bind or empty search)

#$DEBUG=0;
# parse the output to get the node and the value
my $user="";
&log($SCRIPT,"- parsing raw output, \$filter($filter)");
if(@output){
  my $l=-1;# count line to cope with continuing lines # TODO: can be ommited if using $a[-1]=b
  foreach my $line (@output){
    $l++;
    &log($SCRIPT,"--------------------------------------------($l)");
    chomp($line);
    if($line ne ""){
      &log($SCRIPT,"- raw line: ($line)");
      # get the node
      if($filter eq "$dn_prefix=*"){
        &log($SCRIPT,"- \$filter='$filter' \$dn_prefix='$dn_prefix'");
        # find the first word after uid= 'nongreedy'
        # 'dn: uid=cipadmin, ou=user, ou=cipux, o=$CUSTOMER, o=cipworx'
        if($line=~m/^dn: $dn_prefix\=(\S+?)\,/){
          &log($SCRIPT,"found first word");
          $user=$1;
          $user=~s/^\s|\s$//;# del leading trailing space
          &log($SCRIPT,"-> FOUND NODE '$user' ");
          $values{$user}="x";
        }
      }
      # get the value(s)
      my $get="";
      if($line=~m/$entry\:/){
        &log($SCRIPT,"- ldap entry line: ($line)");
        if($line=~m/$entry\:\:/){# this is a base64 enc string!
	  &log($SCRIPT,"- Found base64 encoding");
          $line=~m/$entry\:\: (.+)/;
          $get=&decode_base64($1);
        }else{# normal string
	  &log($SCRIPT,"- Found normal encoding ($l)");
            $line=~m/$entry\: (.+)/;
            $get=$1;
            chomp($get);
        }
        &log($SCRIPT,"-> ldap entry value ($get)");

=pod

        # Its possible to get more than 1 value for 1 node:
        #     value1\n
        #     value2\n
        #     ....
        # Its possible to get more than 1 value for every nodes:
        #     node1 \t value1 \t value2 \t value3 \t value4 ...\n
        #     node2 \t value1 \t value2 \t value3 \t value4 ...\n
        #     ...

=cut

        # META strange!!
        if($filter eq "$dn_prefix=*"){# there are more then one node ..
          &log($SCRIPT,"- there are more then one node");
          if(defined($get)){
          #  $values{$user}="";
          #}
            if(defined($values{$user})){
	      if($values{$user} ne "x" and $values{$user} ne ""){
		$values{$user}.="\t$get";# add the next value with \n
	      }else{# ok, its the first value
		$values{$user}=$get;
	      }
	    }
          }
        }else{# this is only one node($user) but it could have
              # more then one value
          &log($SCRIPT,"- this is only one node($user), but it could have more then one value ");
          #$value=$get;
          if($get ne ""){
            push(@values,$get);
            &log($SCRIPT,"- push single value ($get) to \@values");
          }
        }
      }elsif($line =~m/dn:/){
        &log($SCRIPT,"- skip dn: line, its not a continues line");
      }elsif($line =~ /^\s{1}/){ # continous entry -> man ldif(5
	$line =~ s/^\s{1}//;
        &log($SCRIPT,"- found SPACE at beginning of continous line ($line)");
	$values[-1] .=$line if defined($values[-1]);
	$values{$user} .=$line if defined($values{$user});
      }elsif($line =~ /^\t{1}/){ # continous entry -> man ldif(5)
	$line =~ s/^\t{1}//;
        &log($SCRIPT,"- found TAB at beginning of continous line ($line)");
	$values[-1] .=$line if defined($values[-1]);
	$values{$user} .=$line if defined($values{$user});
      }else{
	# TODO: check, if this else statement is neccessary, because
	#   the to above (better) elsif statments where inserted
	#   (also could be done easier, right?)
        &log($SCRIPT,"- no ldap entry line ($line)");
        # line begin not with an entry considered
        # as continuation of former line
        my $c = $l-2;# warum 2 ?
        if($c > 0){
          # the first line should never be have a continuation
          $line=~s/^\s+//g; #delete leading space, trailing space ok
          &log($SCRIPT,"-> ldap entry value line($l) continues($c) with ($line)");
          if(defined($line) and defined($values[$c])){
            $values[$c] = $values[$c].$line;
            $get="";
          }
        }
      }
    }
  }
}

# +========================================================================+
$PART=              "Output";#                                             |
# +========================================================================+

=pod

=head2 Output formats

=cut

&log($SCRIPT,"- OUT:");
if(@output){
  if($filter eq "$dn_prefix=*"){# we have a hash for multiple value per line
    foreach my $value (keys %values){
      if(defined($value)){
        if(defined($values{$value})){
          &log($SCRIPT,"- multi: $value\t$values{$value}");
          print "$value\t$values{$value}\n";

=pod

 value TAB value
 value TAB values

=cut

        }
      }
    }
  }else{
    foreach my $value (@values){
      if(defined($value)){
        if(defined($opt{v})){# only one cipuxVariable
	  if($value =~ m/$opt{v}=/){
	    $value =~ s/$opt{v}=//g;
	    print "$value\n";
	  }
	  &log($SCRIPT,"- single cipuxVariable: $value");
	}else{
	  print "$value\n";
	  &log($SCRIPT,"- single: $value");
	}
      }

=pod

  value

=cut

    }
  }
  # exit, ok
  exit(0);
}else{
  # exit, error 2
  exit(2);
}

exit(0);


# +========================================================================+
# |                  Subroutines                                           |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

#============================================================|

print "-"x76;
print "\n";
print "$BRAND-$PROJECT-$SCRIPT.pl-$VERSION \n\n";
print "USAGE:\n\n";
print "       $SCRIPT.pl [OPTIONS] -e <ENTRY>\n\n";
print "Options: (exact only one is mandatory)\n";
print "  (01) -u <UCN>  :get user value                   \n";
print "  (02) -c <CCN>  :get course value                 \n";
print "  (03) -m <MCN>  :get machine value                \n";
print "  (04) -r <RCN>  :get room value                   \n";
print "  (05) -t <TCN>  :get hardware type value          \n";
print "  (06) -i <ICN>  :get image value                  \n";
print "  (07) -s <SCN>  :get image slot value             \n";
print "  (08) -y <YCN>  :get admin value                  \n";
print "  (09) -x <XCN>  :get configuration value          \n";
print "  (10) -U        :get value for all users          \n";
print "  (11) -C        :get value for all courses        \n";
print "  (12) -M        :get value for all machines       \n";
print "  (13) -R        :get value for all rooms          \n";
print "  (14) -T        :get value for all hardware types \n";
print "  (15) -I        :get value for all images         \n";
print "  (16) -S        :get value for all image slots    \n";
print "  (17) -Y        :get value for all admins         \n";
print "  (18) -X        :get value for all configuration  \n\n";
print "Axilliary Options:\n";
print "  (19) -h        :get this help page\n";
print "  (20) -v        :get value for cipuxVariable\n";
print "UCN = U+CN = U=user + CN = canonical name, for example: a login\n";
print "-"x76;
print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

cipux_set_value

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
