#!/usr/bin/perl -w
# +=======================================================================+
# || cipux_maint_diagnostic                                              ||
# ||                                                                     ||
# || This script runs several tests against  Test.pm and tries to        ||
# || check if the CipUX installation is ok.                              ||
# ||                                                                     ||
# ||                                                                     ||
# || (c) 2005 Christian Kuelker                                          ||
# ||                                                                     ||
# || License: GPLv2 (GPL version 2 or later)                             ||
# ||                                                                     ||
# +=======================================================================+

use Test;
use strict;

my $cipux_conf = "/etc/cipux/cipux.conf";
my $cipux_schema = "/etc/ldap/schema/cipux.schema";

BEGIN { plan tests => 12 }

print "# This script will test the CipUX installation.\n";
print "# If somthing is not ok, follow the instruction in brackets: [...] if any\n";
print "#\n";
print "#\n";

print "# exists  $cipux_conf? => ";
ok(-e $cipux_conf); # [please install the cipux-cibot package!]

print "# is it readable? => ";
ok(-r $cipux_conf); # [please install the cipux-cibot package!]

print "# is the Ldap_Password set? => ";
my $set_pwd     = `grep Ldap_Password $cipux_conf`;
my $default_pwd = "Ldap_Passeword=secret";
ok($set_pwd ne $default_pwd);# [please set the correct root password at Ldap_Password in $cipux_conf]

print "# existse $cipux_schema? => ";
ok(-e $cipux_schema); # [please install the cipux-common package!]

my $include = `grep $cipux_schema /etc/ldap/slapd.conf`;
print "# is the cipux.schema in slapd.conf included? => ";
ok($include ne "");# [please edit /etc/ldap/slapd.conf and add the line: include $cipux_schema]

my $slapd = `ps ax|grep slapd|grep -v grep`;
print "# is the LDAP server running? => ";
ok($slapd); # [ please start slapd: /etc/init.d/slapd start ]

my @modules = qw(CipUX::CiBot CipUX::Task CipUX::Conversion CipUX::Cat);
foreach my $m (@modules){
 print "# do the module $m load without warnings? => ";
 my $load = `perl -e 'use $m;' 2>&1`;
 print "\n# ERROR: $load\n" if $load ne "";
 ok($load eq ""); # [install a new version of the module!]
}

print "# do the CipUX::CiBot module load with require at once? => ";
ok(eval("require CipUX::CiBot;")); # [please install an new version of cipux-cibot!]

print "# do the CipUX::Cat module load with require at once? => ";
ok(eval("require CipUX::Cat;")); # [please install an new version of cipux-cat-webmin!]

print "# do the CipUX::Task module load with require at once? => ";
ok(eval("require CipUX::Task;")); # [please install an new version of cipux-cibot!]

print "# do the CipUX::Conversion module load with require at once? => ";
ok(eval("require CipUX::Conversion;")); # [please install an new version of cipux-cibot!]

print "# is the cipux_task_list_users script installed? => ";
ok(-e "/usr/bin/cipux_task_list_users");

print "# is the cipux_task_list_users script executable? => ";
ok(-x "/usr/bin/cipux_task_list_users");

print "# get we some ouput? => ";
my @userlist = `/usr/bin/cipux_task_list_users`;
ok($#userlist > 0);

print "# is the cipux_task_list_courses script installed? => ";
ok(-e "/usr/bin/cipux_task_list_courses");

print "# is the cipux_task_list_courses script executable? => ";
ok(-x "/usr/bin/cipux_task_list_courses");

print "# get we some ouput? => ";
my @courselist = `/usr/bin/cipux_task_list_courses`;
ok($#userlist > 0);

my %userlist = ();
foreach my $u (@userlist){
  chomp($u);
  $userlist{$u}=1;
  print "USER $u\n";
}
my %courselist = ();
foreach my $c (@courselist){
  chomp($c);
  $courselist{$c}=1;
  print "COURSE $c\n";
}

print "# is the user-node   cipadmin  there? => ";
ok($userlist{cipadmin}); # [please run cipux_setup_ldap_objects!]

print "# is the group-node  cipadmin  there? => ";
ok($courselist{cipadmin}); # [please run cipux_setup_ldap_objects!]

print "# is the user-node   teacher   there? => ";
ok($userlist{teacher}); # [please run cipux_setup_ldap_objects!]

print "# is the group-node  teacher   there? => ";
ok($courselist{teacher}); # [please run cipux_setup_ldap_objects!]

print "# is the user-node   student   there? => ";
ok($userlist{student}); # [please run cipux_setup_ldap_objects!]

print "# is the group-node  student   there? => ";
ok($courselist{student}); # [please run cipux_setup_ldap_objects!]

print "# is the user-node   guest     there? => ";
ok($userlist{guest}); # [please run cipux_setup_ldap_objects!]

print "# is the group-node  examinee  there? => ";
ok($courselist{guest}); # [please run cipux_setup_ldap_objects!]

print "# is the user-node   examinee  there? => ";
ok($userlist{examinee}); # [please run cipux_setup_ldap_objects!]

print "# is the group-node 'cipadmin' there? => ";
ok($courselist{examinee}); # [please run cipux_setup_ldap_objects!]


print "# is the CipUX Configuration there? => ";
my $cipux_producer=`cipux_get_value -x CAT -e cipuxVariable|grep Cipux_Producer`;
chomp($cipux_producer);
$cipux_producer =~s/Cipux_Producer=//g;
ok($cipux_producer eq "www.cipworx.org");# [The CipUX Configuration LDAP nodes are missing!]

exit(0);

=pod

=head1 NAME

cipux_main_diagnostic - Version <[CIBOT::VERSION]>  2005-08-12 CipUX

=head1 SYNOPSIS

  cipux_main_diagnostic 

=head1 ABSTRACT

This script runs several tests against  Test.pm and tries to       
check if the CipUX installation is ok.                              

=head1 REQUIREMENTS

  CipUX::CiBot
  CipUX::Task
  CipUX::Conversion
  CipUX::Cat

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
