#!/usr/bin/perl -w
# *========================================================================*
# || cipux_set_quota                                                      ||
# || This script set the user hard quota and on a computed basis also the ||
# || soft quota for a given UidNumber (=user).                            ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005 by Christian Kuelker all Rights reserved! ||
# || Licence:   GPLv2 (GPL version 2 or later)                            ||
# ||                                                                      ||
# || Revision:  2005-08-05                                                ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_set_quota - Version <[CIBOT::VERSION]>  2005-03-30 CipUX

=head1 SYNOPSIS

  cipux_set_quota <Filesystem> <UidNumber> <Size> 0
  cipux_set_quota <Filesystem> <GidNumber> <Size> 1
  cipux_set_quota -h
  cipux_set_quota --help

    filesystem : example /home
    idNumber   : uidNumber if type = 0
               : or gidNumber if type = 1
    size       : in blocks
    type       :  1 = group, 0 = user

Example

 * sets quota for user:
 cipux_set_quota '/home' 10024 100000 0

=head1 ABSTRACT

This script sets quota for a idNumber on a given fielsystem.
The idNumber may be UidNumber for type = 0 aka users or
GidNumber for groups if type = 1.

=head1 REQUIREMENTS

  CipUX::CiBot

=cut

package cipux_set_quota;

use CipUX::CiBot;
use strict;

# global variables

my $SCRIPT="cipux_set_quota";

# init logging
&log('cipux_set_quota',"="x76);
my $cml = join(' ',@ARGV);
&log($SCRIPT," $SCRIPT($cml)");

# test if we run
if($cipux{Cipux_Quota}){
  &log($SCRIPT,"* we use quota => continuing");
}else{
  &log($SCRIPT,"* we use not quota => quitting");
  exit(0);
}

my $filesystem = shift;
my $uidNumber = shift;
my $size = shift;
my $type = shift;

&log($SCRIPT,'cipux_set_quota',"cipux_set_quota($filesystem, $uidNumber, $size, $type)");

# options
&usage if not defined($filesystem);
&usage if not defined($uidNumber);
&usage if not defined($size);
&usage if not defined($type);

if(defined($ARGV[0])){
  if($ARGV[0] eq '-h' or $ARGV[0] eq '--help'){
    &usage;
  }
}

&cibot_cipux_set_quota($filesystem, $uidNumber, $size, $type);

exit(0);

=pod

=head1 FUNCTIONS

=cut

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints a usage message.

=cut

sub usage{

  print "USAGE:\n";
  print "  cipux_set_quota filesystem idNumber size type\n";
  print "\n";
  print "    filesystem : example /home\n";
  print "    idNumber   : uidNumber if type = 0\n";
  print "               : or gidNumber if type = 1\n";
  print "    size       : in blocks\n";
  print "    type       :  1 = group, 0 = user\n\n";
  print "Example:\n";
  print "      * sets quota for user:\n";
  print "        cipux_set_quota '/home' 10024 100000 0\n\n";
  exit(1);
}


#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
