#!/usr/bin/perl -w
# *========================================================================*
# || cipux_task                                                           ||
# || This script is the interface to predefined CipUX tasks               ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2 (GPL version 2 or later)                            ||
# ||                                                                      ||
# || Revision:  2005-08-01                                                ||
# || Build:     "<[CIBOT::BUILD>";                                        ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_task - Version <[CIBOT::VERSION]>  2005-08-01 CipUX

=head1 SYNOPSIS

	cipux_task is NOT to be used directly

=head1 ABSTRACT

This script is the interface to predefined CipUX tasks.


=head1 REQUIREMENTS

       CipUX::CiBot
       File::Basename

=cut


# TODO make this work with strict (see or 11.4. Taking References to Functions)
#use strict;
use CipUX::Task;
use File::Basename;

my $DEBUG = 0;

my $caller = basename($0);
print "* cipux_task called as [$caller]\n" if $DEBUG;
my $mode = "console";
print "* cipux_task mode is   [$mode]\n" if $DEBUG;

if($DEBUG){
  print "* registered tasks:\n";
  foreach (sort keys %task){
    print "- $_($task{$_})\n";  
  }
}

# test for only cipux_task
if($caller ne "cipux_task"){
  print "* Ok, we ar not cipux_task directly\n" if $DEBUG;
}else{
  my $error_msg = "ERROR: cipux_task was invoked directly!";
  $error_msg .= " Use the cipux_task_* scripts instead.\n";
  die $error_msg;
  exit(1);
}

# test vor registered cipux_task_*
if(defined($task{$caller})){
  print "* Ok, this task is registered.\n" if $DEBUG;
}else{
  print "This task is not registered!\n";
  print "Use one of the following:\n";
  foreach (sort keys %task){
    print "  $_\n";
  }  
  die "ERROR: $caller is an unknown task!";
}

# some globals after we are sure the task exists
my ($args_no,@dummy) = split(':',$task{$caller});
print "* the task as [$args_no] arguments\n" if $DEBUG;
my ($man,$aux) = split(',',$args_no,2);
print "* [$man] mandatory and [$aux] optional arguments\n" if $DEBUG;

# and now the command line arguments
my $argv = $#ARGV + 1; # number of arguments
print "* got [$argv] arguments\n" if $DEBUG;

# lets see them ony for DEBUG
if($DEBUG){
  foreach my $arg (@ARGV){
    print "- got arg [$arg]\n";
  }
}

# look if we can help
if(defined $ARGV[0]){ #we have 1 arg
  # test for HELP
  if($ARGV[0] eq '-h' or $ARGV[0] eq '--help'){
    &usage($caller);
    exit(0);
  }  
}

# do some forensic 
if($man > 0 or $aux > 0){# task with args
  if($argv < $man){# to few args
    &usage($caller);
    die "ERROR: need $man arguments, got only $argv!\n";
  }
  if($argv > ($man+$aux)){# to few args
    &usage($caller);
    my $sum = $man + $aux;
    die "ERROR: need only $sum arguments, but got $argv!\n";
  }
}else{
  print "* the task as NO arguments\n" if $DEBUG;
  if($argv > 0){ # to many
    die "ERROR: this script do not need any arguments!\n";   
  }
}

my $args = $mode . " " .join(",",@ARGV);
my $exec = "&$caller($args)";
print "* exec subroutine as: [$exec]\n" if $DEBUG;

# or 11.4. Taking References to Functions
my @my_argv = @ARGV;
unshift(@my_argv,$mode); #adds mode in front of @ARGV
my @returned = $caller->(@my_argv);

  
if($#returned > 0){
  print "TASK RETURN:\n" if $DEBUG;
  foreach (@returned){
    print "$_\n";
  }
}


exit(0);

