# +========================================================================+
# || CipUX::CiBot                                                         ||
# || The CiBot library provides specific functions for CipUX bots and     ||
# || to CAT (CipUX Administration Tool)                                   ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht 1999 - 2005 by Christian Kuelker         ||
# || License: GPLv2 (GPL verson 2 or later)                               ||
# || Revision: 2005-08-02                                                 ||
# +========================================================================+
package CipUX::CiBot;

use 5.008000;
use strict;
use warnings;

use Quota;
use vars qw(%ldap %cipux);

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use CipUX::CiBot ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

#&cibot_task_add_user
our @EXPORT = qw(

                &cibot_acl
                &cibot_parse_userdata_conf
                &cibot_change_userdata_conf

		&cibot_get_cipux_config %cipux
		&cibot_get_ldap_config %ldap
		&cibot_parse_command_file

		&cibot_exec_cmd

		&cibot_object_exist
		&cibot_list
		&list_group
		&cibot_file_is_executable_for_group
		&cibot_enable_app
		&cibot_disable_app
		&log
		&error

		&cibot_actual_date

		&cibot_get_password
		&cibot_random_password
		&cibot_get_uid_list
		&cibot_get_login
		&cibot_get_login_list
		&cibot_get_next_idNumber
		&cibot_get_uidNumber

		&cibot_get_quota
		&cibot_set_quota
		&cibot_auto_softquota

);

our $VERSION = '3.002000';


# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CipUX::CiBot - Perl extension for CipUX

=head1 SYNOPSIS

  use CipUX::CiBot;

=head1 DESCRIPTION

This module provides library functions for serveral
perl scripts from the CiBot family and CAT family
under CipUX project, as a part of CipWorX.

New in Version 3: Support for Skolelinux

B<Features:>

=over

=item * preference variable export

=item * verbose, error, log function language dependent

=item * time, date function

=back

=head2 EXPORT

TODO


=head1 REQUIREMENTS

Configuration Files in /etc/cipux

 system.conf  (which determines the next folder in /etc/cipux)
 several other *.conf *.tpl *.txt in /etc/cipux

Perl modules:

Date::Calc
CGI::Carp
POSIX
Tie::IxHash
Getopt::Std
Getopt::Long
File::Basename
Net::IMAP::Simple
Mail::Header
MIME::Base64
File::Find
Cwd
DirHandle
Digest::MD4

=head1 USAGE

Perl scripts witch would like to use CiBot.pm should have lines
like the following:

  use CipUX::CiBot;

=head1 FUNCTION

The perl module CiBot.pm provides the following functions:

 *
 * textfunctions
 * datefunctions
 * systemfunctions
 * log/error functions

=cut




# ..........................................................................
# . Global VARS
# ..........................................................................

#use Date::Calc qw(Delta_Days Today Day_of_Week);

my $BRAND    = "cipworx";
my $PROJECT  = "cipux";
my $SCRIPT   = "CipUX::CiBot";
my $DEBUG    = 0; # DEBUG, only if the script calls the ldap_config
# from system.conf
my $Language = "en";
#my %cipux = ();
#my %ldap  = ();

&cibot_get_cipux_config("/etc/cipux/system.conf");
&cibot_get_cipux_config("/etc/cipux/cipux.conf");

# ..........................................................................
# . I18n
# ..........................................................................
# language

my %lng=();
my $file = "/etc/cipux/lng/$Language";
open(LNG,$file) or die "Can't open file '$file'";
my @lng=<LNG>;
close(LNG);

foreach my $entry(@lng){
  #my($a,$b)=split("=",$entry);
  # split by non greedy (stingy) =
  $entry =~m/(^.+?)=(.+$)/;# you can use = in language text
  my $a=$1;
  my $b=$2;
  chomp($b);
  $lng{$a}=$b;
  #print "LNG: [$a][$b]\n";
}

# ..........................................................................
# . from here on exported LIB functions
# ..........................................................................

# +========================================================================+
# ||                      cibot_change_userdata_conf                      ||
# +========================================================================+

=pod

=head2 cibot_change_userdata_conf

Uses cibot_parse_userdata_conf, apply changes and
write it back to the file /etc/cipux/cat/userdatat.conf

B<Syntax:>

        &cibot_change_userdata_conf(
          $attribute,
          $data,
          $application
          );

=cut

sub cibot_change_userdata_conf{
  my $attribute   = shift; # cipuxFirstname, cipuxRole, ...
  my $data        = shift; # hidden, visible, editable
  my $application = shift; # mandatory, optional
  #print "&cibot_change_userdata_conf($attribute,$data,$application);";

  &log($SCRIPT,"&cibot_change_userdata_conf($attribute,$data,$application);");

  my ($d_href, $a_href, $remarks_aref) = &cibot_parse_userdata_conf;
  my @data = ();
  foreach (sort keys %$d_href){
    &log($SCRIPT,"- parse [$_]");
    if($attribute eq $_){
      push(@data,"$_ $data $application");
    }else{
      push(@data,"$_ $$d_href{$_} $$a_href{$_}")
    }
  }
  my $file = "/etc/cipux/cat/userdata.conf";
  open(F,">$file")
    or die "Can't open '$file' for writing!\n";
    foreach (@$remarks_aref){
      chomp();
      print F "$_\n";
    }
    foreach (@data){
      &log($SCRIPT,"- write [$_]");
      print F "$_\n";
    }
  close(F);
}

# +========================================================================+
# ||                      cibot_parse_userdata_conf                                 ||
# +========================================================================+

=pod

=head2 cibot_parse_userdata_conf

Parses the file /etc/cipux/cat/userdatat.conf and returns
two hash references. One for the userselfadmin.cgi
the other for the application form.

B<Syntax:>

        my(
           $userdata_href,
           $user_application_form_href
           ) = &cibot_parse_userdata_conf();

        my(
           $userdata_href,
           $user_application_form_href,
           $remarks_aref
           ) = &cibot_parse_userdata_conf();

=cut

sub cibot_parse_userdata_conf{
  #my @config = ();
  my $config_file = "/etc/cipux/cat/userdata.conf";
  my @config_line = `cat $config_file`;
  my $userdata_href = {};
  my $user_application_form_href = {};
  my @remarks = ();

  &log($SCRIPT,"- use config_file [$config_file]");

  foreach (@config_line){
    chomp();
    &log($SCRIPT, "- config file line [$_]");
    # if the line do not start with # or empty
    if(m/^\s*[a-z0-9A-Z]+/){
      s/\t/ /g;
      s/^\s+//g;
      s/\s+$//g;

      #push(@config,$_);
      &log($SCRIPT,"- add config parameter [$_]");
      my ($cnf,$usa,$uaf) = split(/\s+/);
      $$userdata_href{$cnf} = $usa;
      $$user_application_form_href{$cnf} = $uaf;
      &log($SCRIPT,"- add config [$cnf] value [$usa]");
      &log($SCRIPT,"- add config [$cnf] value [$uaf]");
    }elsif(m/^#/){# save the remarks
      push(@remarks,$_);
    }
  }
  return $userdata_href,$user_application_form_href,\@remarks ;
}

# +========================================================================+
# ||                      cibot_cal                                       ||
# +========================================================================+

=pod

=head2 cibot_acl;

Test the rights for the remote user to access the script.
If the acess is granted the value 'ok' is returned. If not
a message is returned. Ths may change in the future.
As a second value the role will be returned.

B<Syntax:>

        my($result,$role) = &cibot_acl($SCRIPT,$remote_user);

=cut

sub cibot_acl{
  my $script      = shift;
  my $remote_user = shift;

  &log($SCRIPT,"&cibot_acl($script,$remote_user);");

  my $role = "";
  my $message = "";

  if($remote_user eq 'root'){
    $role = 'admin';
  }elsif($remote_user eq 'cipadmin'){
    $role = 'admin';
  }else{
    #TODO: make a CAT or CipUX subroutine
    $role = `cipux_get_value -c $remote_user -e cipuxRole`;
    chomp($role);
  }
  $role = 'guest' if not defined($role);
  $role = 'guest' if $role eq '';
  &log($SCRIPT,"- role [$role]");

  my $r = $role;
  # Cat_ACL_Student = index.cgi self.cgi documentation.cgi faq.cgi
  my $v = "Cat_Acl_" . ucfirst($script);
  $v =~ s/\.cgi$//g;
  &log($SCRIPT,"- variable [$v]");
  my $acl = $ldap{CAT}{$v};
  &log($SCRIPT,"- acl [$acl]");
  &log($SCRIPT,"      [Cat_Acl_Userselfadmin]");

  if($acl =~ m/$role/){
    $message = "ok";
  }else{
    $message = "denied";
    #$text('access_denied_explaination',$remote_user,$role);
  }

  &log($SCRIPT,"- access [$message]");
  return $message,$role;
}

# +========================================================================+
# ||                      cibot_get_cipux_config                          ||
# +========================================================================+

=pod

=head2 cibot_get_cipux_config;

Get variables and there values from a configuration file and
export them to a different perl script.

B<Syntax:>

	$conf_hashref = &cibot_get_cipux_config($config_file);

B<Requirements:>

Perl scripts witch would like to use cibot_get_cipux_config

  use CipUX::CiBot;

  &cibot_get_cipux_config($config_file);

Then the variable %cipux will be filled
with config variables from the $config_file.

The config file, for example:

 /etc/cipux/default/cipux.conf

should have lines like the following:

 # Comment
 var1 = value1
 var2=value2
 var3 =value3
 var4= value4
 [section]
  var5 = value5

Lines starting with # or [ are ignored.

The location of the config_file can be determined by
reading: /etc/cipux/system.conf which contains the
line:

 custumer = default

od

 customer = skolelinux

or some other value. The path will be constructed as:

 /etc/cipux/$customer/cipux.conf


=cut

sub cibot_get_cipux_config{

  my $file = shift;

  &log($SCRIPT,"&cibot_get_cipux_config($file)");

  if(-e $file){
    open(CONF,"<$file") or error('0001',$file);# ERROR: Can't open preference file
    while(<CONF>){
      if(/^#.*/ or /^\[.*/ or /^\s+$/){
        &log($SCRIPT,"- comment, section, spaced or empty line");
      }else{
        &log($SCRIPT,"- parse config line ...");
        chomp;                 # kill Newline/ entfernt Newline
        s/^\s+//;              # no leading spaces/ keine fuehrende Leerzeichen
        s/#.*//;               # no comments/ keine Kommentare
        s/\s+$//;              # no trailing spaces/ keine folgenden Leerzeichen
        next unless length;    # something forgotten?/ war da noch was?
        if(/=/){
          # variables x= y / Variablen  X = Y
          my ($variable,$value)=split(/\s*=\s*/,$_,2);
	  chomp($value);
	  if(defined($variable) and defined($value)){
	    $cipux{$variable}=$value;
              &log($SCRIPT,"-> variable [$variable] = value [$value]");
          }
        }
      }
    }
    close(CONF);
    return $cipux{Customer} if defined($cipux{Customer});
  }else{
     &error('001',$file);
  }
}

# +========================================================================+
# ||                      cibot_get_ldap_config                           ||
# +========================================================================+

=pod

=head2 cibot_get_ldap_config;

Get variables and there values from the LDAP server
and returns a hash reference and export this hash.

B<Syntax:>

	$conf_hashref = &cibot_get_ldap_config();

B<Requirements:>

Perl scripts which would like to use cibot_get_ldap_config

  use CipUX::CiBot;

  &cibot_get_ldap_config();

The variable (and therefore the hash referenz)
will be constructed as follows:

 %ldap = ();
 $ldap{type}{varname}="value";

Example:

 type = cat
 varname = Cat_Feature_Useradmin
 value=1


=cut

sub cibot_get_ldap_config{

  &log($SCRIPT,"&cibot_get_ldap_config");

  my $cmd = "/usr/bin/cipux_get_value -X -e cipuxVariable";

  # execute the command
  my $pid=open(GETVALUE,"$cmd |")
    or &error('0172',$cmd); # Can't execute get_value.pl command:
  my @raw_config=<GETVALUE>;
  my $ok=close(GETVALUE);
  # Error occured while executing get_value.pl (no PID):
  error('0173',$cmd) if !defined($pid);
  # Error occured while executing get_value.pl (no bind or empty search):
  error('0174',"($pid)$cmd") if !$ok;

  foreach (@raw_config){
    # one line per LDAP object:
    # [CipUX     Cipux_Producer=www.cipworx.org  debug=1]
    # [CAT       Cipux_Producer=www.cipworx.org  useradmin=1]
    chomp;
    &log($SCRIPT,"- ldap config entry: [$_]");
    my ($type,$var_value)=split("\t",$_,2);
    my @var_value = split("\t",$var_value);
    foreach my $pair (@var_value){
      my ($var, $value) = split(/=/,$pair,2);
      $var   =~ s/^\s*//g;
      $var   =~ s/\s*$//g;
      $value = '' if not defined($value);
      $value =~ s/^\s*//g ;
      $value =~ s/\s*$//g ;
      $ldap{$type}{$var}=$value;
      &log($SCRIPT,"- ldap config: ldap[$type][$var]=[$value]");
    }
  }
}


# +========================================================================+
# ||                      cibot_parse_command_file                          ||
# +========================================================================+

=pod

=head2 cibot_parse_command_file

Parse a setup file like add.conf or del.conf. Then replace special tags
(say for example <ldap>cipuxFirstname</ldap>) with LDAP attributes
by there LDAP values and give this file back as a reference to an array.

Additionally the string $login (and if given $password) will be
replaced by its values.

Furthermore all variabels from cipux.conf and system.conf (TODO)
will be replaced like this:

 <cipux>Ldap_Host</cipux>

will be replaced by:

 localhost

and

 <system>Customer</system>

will be replaced by:

 default

for example.

B<Syntax:>

  my $file_array_ref = &cibot_parse_command_file($object,
                                                 $config_file,
					         $login
					         [,$user_password]);

Example for:

  my $object        = "User";
  my $config_file   = "/etc/cipux/default/del.conf
  my $login         = 'chkuelker';

=cut

sub cibot_parse_command_file {

  my $object        = shift; # User, Course, Mashine
  my $config_file   = shift; # path+filename
  my $login         = shift; # chkuelker
  my $user_password = shift; # secret
#  my $cipux         = shift; # hash_ref to %cipux (sorry)

  if(defined($user_password)){
    &log($SCRIPT,"&cibot_parse_command_file($object,$config_file,$login,$user_password)");
  }else{
    &log($SCRIPT,"&cibot_parse_command_file($object,$config_file,$login)");
  }
  &log($SCRIPT,"- object     : [$object]");
  &log($SCRIPT,"- config file: [$config_file]");

  my @commands=();

  # open config file:
  my $pid = open(CONFIG,$config_file)
    # <SCRIPT>: Can't open config file:
    or error('0127',$config_file);
  my @config=<CONFIG>;
  my $ok=close(CONFIG);
  # <SCRIPT>: Error executing open config file:
  &error('0128',$config_file) if !defined($pid);
  # <SCRIPT>: Opening config file not successful:
  &error('0129',$config_file) if !$ok;

  foreach (@config){
    &log($SCRIPT,"-  raw line [$_]");
    chomp();
    s/\#.*//g; # no commnets
    s/^\s+//g; # no leading spaces
    s/\s+$//g; # no trailing spaces
    if($_ ne ''){
      &log($SCRIPT,"- parse config line: [$_]");
      my($cmd_object,$command) = split(/\s+=\s+/,$_);
      # if not comment, empty or space line:
      push(@commands,$command) if $cmd_object eq $object;
    }
  }

  my %attributes=();# collected ldap attributes
  #my %cipux_vars=();# collectes cipux.conf variables
  #my %system_vars=();# collectes system.conf variables
  foreach(@commands){
    # replace $1 from <replace>$1</replace>
    while(m{<ldap>((?:(?!<ldap>).)*)</ldap>}g){# META: \s+ neccessary?
      $attributes{$1}++;
    }
    while(m{<cipux>((?:(?!<cipux>).)*)</cipux>}g){# META: \s+ neccessary?
      if(defined($cipux{$1})){
	my $r = $cipux{$1};
	&log($SCRIPT,"- found fragment [$r]");
	s\<cipux>.*</cipux>\$r\g;
      }else{
	&error('0186',$1);# setup.pl: Some <cipux>variable</cipux> can't be replaced: Probably the config file add.conf or del.conf needs to be checked. Please have a look at cipux.conf for the right spelling. The miss spelled variable is:

      }
    }
    #while(m{<system>((?:(?!<system>).)*)</rsystem>}g){# META: \s+ neccessary?
    #  s/(<system>?:?!<system>.*</system>)/$cipux{$1}/g;
    #}
  }

  # META: may check if all attributes ar valid attributes?

  # get value for every object attribute:
  foreach (keys %attributes){
    chomp();
    &log($SCRIPT,"- attribute: [$_] [$attributes{$_}]");
    # open get attribute for user:
    my $switch = "-u";
    $switch = "-c" if $object eq "Course";
    my $cmd =  "/usr/bin/cipux_get_value -e '$_' $switch $login";
    &log($SCRIPT,"- Cmd get_value: [$cmd]");
    my $pid = open(CMD,"$cmd |")
      # <SCRIPT>: Can't execute command:
      or error('0130',$cmd);
    $attributes{$_}=<CMD>;
    my $ok=close(CMD);
    # <SCRIPT>: Error while executing command:
    &error('0131',$cmd) if !defined($pid);
    # <SCRIPT>: Executing command not successful:
    &error('0132',$cmd) if !$ok;
    chomp($attributes{$_});
    &log($SCRIPT,"attribute:($_)$attributes{$_}");
  }

  # replace the attributes and $login, $password
  foreach my $cmd (@commands){
    foreach my $attr (keys %attributes){
       chomp($attr);
      #print "A:($attr)$attributes{$attr}\n";
      my $search = '<ldap>'.$attr.'</ldap>';
      $cmd =~ s/$search/$attributes{$attr}/ge;
      $cmd =~ s/\$login/$login/ge;
      if(defined($user_password)){
	$cmd =~ s/\$password/$user_password/ge;
      }
    }
    &log($SCRIPT,"- CMD: [$cmd]");
  }

  &log($SCRIPT,"=> return array_ref \@commands [\@commands]");
  return \@commands;

}

# +========================================================================+
# ||                      External Commands, Processes:                   ||
# +========================================================================+

sub cibot_exec_cmd{

  my $mode = shift;  # mode=html,term
  my $cmd = shift;
  my $hint = shift;
  my $script = caller();

  if(not defined($mode)){
    if(defined($ENV{TERM})){
      $mode = 'term';
    }else{
      $mode = 'html';
    }
  }
  &log($SCRIPT,"- mode  [$mode]");

  $script =~ s/_/\./g; # because packages cant be named: useradmin.cgi

  &log($SCRIPT,"- cmd = [$cmd]");
  &log($script,"- cmd = [$cmd]");

  my $o = ""; # out string

  my $error = "($cmd) $hint";
  $error = "($cmd)<br> <b>$hint</b>" if $mode eq 'html';

  # execute the command
  my $pid=open(CMD,"$cmd |")
    or &error('0190',$error); # error::0190=Can NOT execute the command:
  my @out=<CMD>;
  my $ok=close(CMD);
  # error::0191=An error (no process ID) eccured while executing the command:
  error('0191',$error) if !defined($pid);
  foreach (@out){
    if($mode eq 'html'){
      $o .= $_ .'<br>';
    }else{
      $o .= $_;
    }
  }
  # error::0192=An error (the return value is not OK [this could stand for: no bind, empty search, ... or somthing else, depending on the command]) occured while executing the command:
  $error = "PID = [$pid]\n CMD = [$cmd]\n OUTPUT FROIM THE COMMAND [$o]\n HINT $hint";
  $error = "<ul>
              <li>PID = [$pid]</li>
              <li>CMD = [$cmd]</li>
              <li><b>OUTPUT FROM THE COMMAND:</b></li>
              <li>[$o]</li>
              <li><b>$hint</b></li>
            </ul>" if $mode eq 'html';
  error('0192',$error) if !$ok; # error::0192=An error (the return value is not OK) occured while executing the command:

  if(scalar(@out) >0){# if there is something
    if($mode eq 'html'){
      print "<hr noshade><ul>";
      foreach (@out){
	print "<li>$_</li>";
    }
      print "</ul><hr noshade>";
    }
    return \@out;
  }
}

# +========================================================================+
# ||                            text                                      ||
# +========================================================================+

sub cibot_list{

  # Interface to get_value.pl (Uppercase letters)
  # Example:
  # cibot_list('C',gidNumber); returns all gid numbers in an hash
  # cibot_list('C',memberUid); a hash over all gid, returns one string value
  # cibot_list('C',memberUid,1); a hash over all gid, retruns an arry_ref
  # plan to change this in future

  my $kind        = shift;
  my $attr        = shift;
  my $return_type = shift;

  if(defined($return_type)){
    &log($SCRIPT,"&cibot_list($kind,$attr,$return_type)")
  }else{
    &log($SCRIPT,"&cibot_list($kind,$attr)");
  }



  # /usr/bin/cipux_get_value -U -e uidNumber";
  my $cmd = "/usr/bin/cipux_get_value -".$kind." -e $attr";
  &log($SCRIPT,"- using cmd: [$cmd]");
  my %user=();
  open(COMMAND,"$cmd |") or die print "Can't execute '$cmd'! \n";
  while(<COMMAND>){
    if($_ ne ""){
      chomp();
      &log($SCRIPT,"- get raw line [$_]");
      if(not defined($return_type)){# old: on string per key
	my ($login,$answer) = split("\t",$_);
	if($login ne "root"){
	  $user{$login}=$answer;
          &log($SCRIPT,"- old kind  [$kind]")   if defined($kind);
          &log($SCRIPT,"- old login [$login]")  if defined($login);
          &log($SCRIPT,"- <= attr   [$attr]")   if defined($attr);
          &log($SCRIPT,"- <= answer [$answer]") if defined($answer)
	  #&log($SCRIPT,"- old $kind [$login] <= $attr [$answer]");
	}
      }else{# new one aray per key
	my ($login,@answer) = split("\t",$_);
	if($login ne "root"){
	  $user{$login}=[ @answer ];
	  &log($SCRIPT,"- new $kind [$login] <= $attr [@answer]");
	}
      }
    }
  }
  close(COMMAND);

  &log($SCRIPT,"=> retrun hash_ref \%user [\%user]");
  return \%user;

}

#TODO: DELETE
sub list_group {

 my $kind = shift;# school_class, ....

 &log($SCRIPT,"&list_group('$kind')");

 my $cmd = "/usr/bin/cipux_get_value -C -e groupType";
 &log($SCRIPT,"- using cmd: [$cmd]");
 my %course=();
 open(COMMAND,"$cmd |") or die print "Can't execute '$cmd'! \n";
 while(<COMMAND>){
   if($_ ne ""){
     chomp();
     my ($course,$type) = split("\t",$_);
     $course{$course}=$type;
     &log($SCRIPT,"- $kind [$course] <= [$type]");
   }
 }
 close(COMMAND);
 my @answer=();
 foreach (sort keys %course){
   if($course{$_} eq $kind){
     push(@answer,$_);
     &log($SCRIPT,"- return [$course{$_}][$_] ");
   }
 }

 &log($SCRIPT,"=> return aash_ref answer [\@answer]");
 return \@answer;

}

# +========================================================================+
# || cibot_file_is_executable_for_group                                   ||
# +========================================================================+
sub cibot_file_is_executable_for_group{

  my $file = shift;
  my $group =shift;

  &log($SCRIPT,"&cibot_file_is_executable_for_group($file,$group)");

  my $status = 1; # 1 = undefined, 0 = access denied by group
  my $cmd = "getfacl --tabular --absolute-names --all-effective";
  my @out = `getfacl --tabular --absolute-names --all-effective $file;`;
  foreach (@out){
    if(not /^\s*#|^\s*$/){
      chomp();
      &log($SCRIPT,"- getfacl line: [$_]");
      if(/$group/){ # we test only of the group is there, because, we
	# assume: if its there the the ACL will restrict the right, if its not there, then
	#the normal access rights are valid
	&log($SCRIPT," -> found group in getfacl line: [$group]");
	$status = 0;
      }
    }
  }

  &log($SCRIPT,"=> return \$status [$status]");
  return $status;

}

# +========================================================================+
# || cibot_enable_app                                                     ||
# +========================================================================+

sub cibot_enable_app{

  my $file = shift;
  my $group = shift;

  &log($SCRIPT,"&cibot_enable_app($file,$group)");

  my @out = `setfacl -x g:$group $file`;

  &log($SCRIPT,"=> return array_ref out [\@out]");
  return \@out;

}
# +========================================================================+
# || cibot_disable_app                                                     ||
# +========================================================================+

sub cibot_disable_app{

  my $file = shift;
  my $group = shift;

  &log($SCRIPT,"&cibot_disable_app($file,$group)");

  my @out = `setfacl -m g:$group:- $file`;

  &log($SCRIPT,"=> return array_ref out [\@out]");
  return \@out;

}


# +========================================================================+
# || log (logging if DEBUG)                                               ||
# +========================================================================+

sub log{

  my $script = shift;
  my $message = shift;

  if($cipux{Cipux_Debug}){
    my $path = "/var/log/cipux";
    if(!-d $path){mkdir($path,700)}
    my $logfile=$path."/".$script.".log";
    open(DEBUG,">>$logfile") or die "Can't open logfile '$logfile'";
    #my $date = `date`;
    #chomp($date);
    print DEBUG "$script: $message\n";
    close(DEBUG);
  }

}

# +========================================================================+
# ||                            error                                     ||
# +========================================================================+

=pod

=head2 error

Prints a language dependend error message on STDOUT.
Hint: '0000' is well defined error number

B<Syntax:>

	cibot::error('0000','VALUE_TEXT');

	cibot::error('0000');

=cut

sub error{

  my $c = caller();
  my $e = shift; # error
  my $v = shift; # value

  print "WARNING: e not defined" if not defined($e);
  print "WARNING: c not defined" if not defined($c);
  print "WARNING: v not defined" if not defined($v);

  my $m = $lng{"error::". $e };
  my $s = "ERROR <$e>: $m";
  $s .= " [$v]" if defined($v);
  $s .= "!";

  &log($SCRIPT,$s);
  &log($c,$s);
  print "$c $s\n";

  exit(1);
}

# ..........................................................................
# date and time functions
# .......................................................................

# +========================================================================+
# ||                      cibot_actual_date                               ||
# +========================================================================+

=pod

=head2 cibot_actual_date

Gives the actual system date as ISO-8601: YYYY-MM-DD

B<Syntax:>

	my $date = &cibot_actual_date();

=cut

sub cibot_actual_date{

  # Use the system function /bin/date
  # Anlagedate der Systemfunktion date
  #open (HANDLE, "/bin/date|");
  #while(<HANDLE>){
  #  $date=$_;
  #}
  #close(HANDLE);

  my ($day, $month, $year)=(localtime)[3,4,5];
  $year=$year+1900;
  $month=$month+1;
  $month=substr("0".$month,-2,2);
  $day=substr("0".$day,-2,2);

  my $date=$year."-".$month."-".$day;

  &log($SCRIPT,"=> return date [$date]");
  return $date;

}

# ..........................................................................
# system functions
# ..........................................................................

# +========================================================================+
# ||                      cibot_get_password                              ||
# +========================================================================+

=pod

=head2 cibot_get_password

Makes passwords from a given string. It will use the programm mkntpwd.
Return values are  CRYPT, lmPassword and ntPaswwort.

B<Syntax:>

  my($crypt,$lmPassword,$ntPaswwort) = &get_password($password);

=cut

sub cibot_get_password{

  my $password = shift;

  &log($SCRIPT,"&cibot_get_password($password)");

  # MD5 password
 #my $random = join '',('.','/',0..9,'A'..'Z','a'..'z')[rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64];
 #
 #my $bsalt = "\$1\$";
 #my $esalt = "\$";
 #my $modsalt = $bsalt.$random.$esalt;

  # crypt password
  my $modsalt = join '', ('.','/',0..9,'A'..'Z','a'..'z')[rand 64,rand 64];
  my $crypt_password = crypt($password, $modsalt);

  my $lm_password = "";
  my $nt_password = "";

  if($cipux{Cipux_Use_Samba} eq "yes"){
    &log($SCRIPT,"- Cipux_Use_Samba=yes");
    # LanManager and NT clear text passwords
    my $ntpwd = `/usr/local/sbin/mkntpwd '$password'`;
    &log($SCRIPT,"=> mkntpwd = [$ntpwd]");

    chomp($lm_password = substr($ntpwd, 0, index($ntpwd, ':')));
    chomp($nt_password = substr($ntpwd, index($ntpwd, ':')+1));
  }

  &log($SCRIPT,"=> retrun crypt password = [$crypt_password]");
  &log($SCRIPT,"=> retrun NT passwort    = [$nt_password]");
  &log($SCRIPT,"=> return LM passwort    = [$lm_password]");
  return $crypt_password,$lm_password,$nt_password;

}


# +========================================================================+
# ||                      cibot_random_password                           ||
# +========================================================================+

=pod

=head2 cibot_random_password

Generate random passowrd

B<Syntax:>

  my $password = &random_password();

=cut

sub cibot_random_password{

  &log($SCRIPT,"&random_apssword()");

  # see cookbook, de <55>
  my @chars = ('A'..'Z','a'..'z',0..9,qw(! @ $ % &));
  my $password = join("",@chars[map{rand@chars}(1..8)]);

  &log($SCRIPT,"=> return \$password [$password]");
  return $password;

}

# +========================================================================+
# ||                      cibot_get_login                                 ||
# +========================================================================+

=pod

=head2 cibot_get_login

Creates and check a new login.
It will add 2 firstname characters and the lastname to
combine a new login. It will then check if this login exsists. Therefor
it searches user, course and machine. (Witch should give as a hashref)
if it exsists, it will add a counter to the login. For example
'chkuelker1'. Then it checks this again. Of course if it exsis,
it will increase the counter. And so on. The return value will be
a valid login.

B<Syntax:>

  my $login = &get_login($loginify_firstname,$loginify_lastname,%userlist);

B<Discussion:>

If the firstname or lastname contains non ASCII characters they will
be substituted by ASCII characters. This called loginify and have to be
done by a function of conversion.pm named: loginify_string
For example a german Umlaut will become 'ae'. Every hyphen will
become a underline: '_'. See there for further discription.

In this function: The lenght
of the login will be restricted to max. 12 characters.

Then the function will check, if a login exists an behalf
of the given reference to a userlist hash. %usrerlist{$user}=$value,
$value > 0 or true, can be 1 or uid.

=cut

sub cibot_get_login{

  # get firstname and lastname
  my $firstname = shift;
  my $lastname = shift;
  my $userlist_ref = shift;

  &log($SCRIPT,"&get_login($firstname,$lastname,$userlist_ref)");

  &log($SCRIPT,"- FN: [$firstname] LN: [$lastname]");

  # cut down lower firstname to 2 char
  my $f_login = lc(substr($firstname,0,2));

  # cut down lower lastname to 10 chars
  my $l_login = lc(substr($lastname,0,10));

  # convert special chars
  # second lc is somtimes importand  -> A1/4 -> A -> a
  # nevertheless its a mistake!
  my $login = lc($f_login.$l_login);

  # Check on behalf of userlist if login exists:
  my $end=0;
  my $count=1;

  &log($SCRIPT," - UL [$userlist_ref]");

  do{

    # 1 do login exists?
    # META: test with ldap
    #my $USER_DIR_PATH="";
    #if(-e "$USER_DIR_PATH/$login"){
    if($userlist_ref->{$login}){# hashref value for existing login true

      &log($SCRIPT,"-  other login: [$userlist_ref->{$login}]");
      # delete old number on the end (if counted before)
      $login =~ s/\d+$//;

      # 3 if yes increase login (wenn ja nehme login --> login1)
      $login = $login . "$count";
      $count++;

      # 4 once again (nochmal von vorne)
    }else{

      # 2 if no, then end (wenn nein dann ende)
      $end=1;
    }
  }until($end);

  # delete space and tab and return from the other
  $lastname=~ s/^\s+|\s+$| |\t|\n//g;
  $firstname=~ s/^\s+|\s+$| |\t|\n//g;

  &log($SCRIPT,"=> return \$login [$login]");
  return $login;

}


# +========================================================================+
# ||                     cibot_get_login_list                             ||
# +========================================================================+

=pod

=head2 cibot_get_login_list

Get a user list. Returns it in a hash reference.

B<Syntax:>

  %userlist = &cibot_get_login_list($array_ref);

  $array_ref: Reference to a command array like

  my @cmds = ("$CiBot_Dir/bin/$CiBot_List_Cmd -U",
	      "$CiBot_Dir/bin/$CiBot_List_Cmd -C",
	      "$CiBot_Dir/bin/$CiBot_List_Cmd -M")

=cut

sub cibot_get_login_list{

  my $command_ref = shift;# array ref
  &log($SCRIPT,"&cibot_get_login_list($command_ref)");

  my %user_list =();

  foreach my $command (@$command_ref){

    # META write error codes!
    my @user_list = `$command`;

      foreach my $user (@user_list){
	$user=lc($user);
	chomp($user);
	&log($SCRIPT,"- user $user");
	$user_list{$user}=1;# META: may be save uid in the future?
      }
  }

  &log($SCRIPT,"=> return hash_ref user_list [\%user_list]");
  return \%user_list;# returns a hash reference

}



# +========================================================================+
# ||                      cibot_get_next_idNumber                         ||
# +========================================================================+


=pod

=head2 cibot_get_next_idNumber

Return value is a valid uidNumber or gidNumber search user, course and machine tree in LDAP.

B<Syntax:>

  my $idNumber = &cibot_get_next_idNumber($type,$kind);

  $type         : u,c,m, ...
  $kind         : uidNumber, gidNumber

B<Discussion:>

=cut

sub cibot_get_next_idNumber{

# my $option      = shift;
# my $attribute   = shift;
# my $start       = shift; # to be implemented to start, to assure uidNumber=gidNumber

 #if(defined($start)){
 #  &log($SCRIPT, "& cibot_get_next_idNumber($option,$attribute,$start)");
 #}else{
 #  &log($SCRIPT, "& cibot_get_next_idNumber($option,$attribute)");
 #}
  &log($SCRIPT, "& cibot_get_next_idNumber()");

  my $u_href = &cibot_list('U','gidNumber');
  my $c_href = &cibot_list('C','gidNumber');
  my $m_href = &cibot_list('M','uidNumber');# only one machine group

  #  my $object_href = &cibot_list(uc($option),$attribute);
  my %merged = (%$u_href,%$c_href,%$m_href);
  my $object_href = \%merged;

  my %idNumber = ();
  foreach my $key(%$object_href){
    if(defined($key)){# TODO: find out why this may not be defined
      if(defined($$object_href{$key})){
        if($$object_href{$key} ne ''){
	  $idNumber{$$object_href{$key}}=1;
	  &log($SCRIPT,"- idNumber [$$object_href{$key}]");
        }
      }
    }
  }

  foreach my $i ($cipux{Pam_UID_Start} .. $cipux{Pam_UID_End}) {
    &log($SCRIPT,"- i [$i]");
    if(not defined($idNumber{$i})){
      #my $hex_uid= sprintf("%x", $i);
      &log($SCRIPT,"- RESULT NEXT NEW ID = [$i]");
      return $i;
    }
  }
  &error('0114');# ERROR: No UID left on device CipUX LDAP
  exit(0);
}

sub to_be_deleted{
  my $start_uid    = shift;# uid to start with
  my $end_uid      = shift;# uid where to end
  my $uid_list_ref = shift;# ref to hash like $uid_list{$uid}=1

  &log($SCRIPT,"&cibot_get_next_uid($start_uid,$end_uid,$uid_list_ref)");

  # TODO for customer skolelinux get uid from LDAP? and save the uid to LDAP

  chomp($start_uid);
  chomp($end_uid);

  # lets start with a uid:
  my $uid = $start_uid;

  &log($SCRIPT,"- uid start: [$start_uid]");
  &log($SCRIPT,"- id end:    [$end_uid]");
  &log($SCRIPT,"- hash_ref:  [$uid_list_ref]");

  # Check on behalf of uid list if uid exists:
  my $end=0;
  do{

    # 1 do uid exists?
    &log($SCRIPT,"- TEST $start_uid -> $end_uid: $uid");
    &log($SCRIPT,"  ($uid_list_ref->{$uid})") if defined ($uid_list_ref->{$uid});

    # hashref value for existing uid true
    if(defined($uid_list_ref->{$uid})){

      # 3 if yes increase login (wenn ja nehme login --> login1)
      $uid++;

      # 4 once again (nochmal von vorne)
    }else{

      # 2 if no, then end (wenn nein dann ende)
      $end=1;
    }
  }until($end);

  # as grouprid we use hex value of gid
  #$_userGroupRid = sprintf("%x", $_userGidNumber);
  #$_userRid  = sprintf("%x", $_userUidNumber);

  #if sombody wants the HEX value, who could it be?
  my $hex_uid= sprintf("%x", $uid);

  if($uid>=$end_uid){

  }else{
    &log($SCRIPT,"- UID GIVEN: $uid ($hex_uid)");
    chomp($uid);
    chomp($hex_uid);
    &log($SCRIPT,"=> return \$uid \$hex_uid [$uid][$hex_uid]");
    return $uid,$hex_uid;
  }

  &log($SCRIPT,"=> return [0]");
  return 0;# newert

}
# +========================================================================+
# ||                      cibot_get_uid_list                              ||
# +========================================================================+


=pod

=head2 cibot_get_uid_list

Return list of uid values (in a hash reference) which are present in
the LDAP server for trees: user, course and machine.

B<Syntax:>

  my $uid_list_ref = &cibot_get_uid_list($user_uid_list_command1,...);

B<Discussion:>

  @user_uid_list:

  user[TAB]uid

  get_value.pl -e uidNumber -U
  get_value.pl -e uidNumber -C
  get_value.pl -e uidNumber -M

=cut

sub cibot_get_uid_list{

  my $commands_ref = shift;# array ref

  &log($SCRIPT,"&cibot_get_uid_list($commands_ref)");

  my %uid_list=();

  foreach my $command (@$commands_ref){

    # META write error codes!
    &log($SCRIPT,"- CMD: [$command]");
    my @user_tab_uid_list = `$command`;

    foreach my $line (@user_tab_uid_list){
      my($object,$uid)  = split("\t",$line);
      if(defined($uid)){
	chomp($uid);
	&log($SCRIPT,"OBJ $object UID '$uid'");
	$uid_list{"$uid"} = '1';
      }
    }

    if($DEBUG){
      foreach my $uid (sort keys %uid_list){
	&log($SCRIPT,"UID LIST: $uid");
      }
    }
  }

  &log($SCRIPT,"=> return hash_ref uid_list[\%uid_list]");
  return \%uid_list;# hash_ref

}

# =========================================================================+
# ||                    Quota                                             ||
# =========================================================================+
sub cibot_set_quota {
  my ($filesystem, $login, $size, $type ) = @_;  # user: type=0, group type=1;
  &log($SCRIPT,"&cibot_get_quota($filesystem, $login, $size, $type)");

  if($type){
    &log($SCRIPT,"- group quota");
  }else{
    &log($SCRIPT,"- user quota");
  }

  my $dev = Quota::getqcarg($filesystem);
  &log($SCRIPT,"- quota dev: $dev");

  &Quota::sync($dev);
  &log($SCRIPT,"- quota sync");

  my ($bc,$bs,$bh,$bt,$ic,$is,$ih,$it) = Quota::query($dev);
  &log($SCRIPT,"-  query: ($bc,$bs,$bh,$bt,$ic,$is,$ih,$it,type) ");

  #$size = $size;
  my $p30 = 0;
  $p30 = int($size/100*30) if $size >0;
  $bs = $size - $p30;
  #$bs = 10 if $size == 0; # to avoid unlimided quota
  $bh = $size;
  $bs = 0 if $size < 0;
  &log($SCRIPT,"-> set quota for [$type] [$login] block soft[$bs]($p30),hard[$bh] inode[$is][$ih] on $dev");
  Quota::setqlim($dev, $login, $bs,$bh, $is,$ih, 0, $type);

}

sub cibot_get_quota{
  my ($filesystem, $login, $type) = @_; # user: type=0, group: type=1
  &log($SCRIPT,"&cibot_get_quota($filesystem, $login, $type)");

  if($type){
    &log($SCRIPT,"- group quota");
  }else{
    &log($SCRIPT,"- user quota");
  }

  my $dev = Quota::getqcarg($filesystem);
  &log($SCRIPT,"- quota dev: $dev");

  &Quota::sync($dev);
  &log($SCRIPT,"- quota syn");

  my ($bc,$bs,$bh,$bt,$ic,$is,$ih,$it) = Quota::query($dev, $login, $type);

  &log($SCRIPT,"=> retrun [$bc,$bs,$bh,$bt,$ic,$is,$ih,$it] = &cibot_get_quota");

  return $bc,$bs,$bh,$bt,$ic,$is,$ih,$it;

}

sub cibot_auto_softquota{
  # &cibot_auto_softquota($size)
  my $size = shift;

  my $s = int($size);
  my $p30 = 0;        # 30 %
  $p30 = int($s/100*30) if $s >0;
  my $r = $s - $p30;

  return int($r);
}


1;
__END__


sub WAHT_SUB_IS_THIS_TODO{
  # TODO for customer skolelinux get uid from LDAP? and save the uid to LDAP

  chomp($start_uid);
  chomp($end_uid);

  # lets start with a uid:
  my $uid = $start_uid;

  &log($SCRIPT,"- UID START $start_uid");
  &log($SCRIPT,"- UID END  $end_uid ");
  &log($SCRIPT,"- HASH REF $uid_list_ref");

  # Check on behalf of uid list if uid exists:
  my $end=0;
  do{

    # 1 do uid exists?
    &log($SCRIPT,"- TEST $start_uid -> $end_uid: $uid");
    &log($SCRIPT,"  ($uid_list_ref->{$uid})") if defined ($uid_list_ref->{$uid});

    # hashref value for existing uid true
    if(defined($uid_list_ref->{$uid})){

      # 3 if yes increase login (wenn ja nehme login --> login1)
      $uid++;

      # 4 once again (nochmal von vorne)
    }else{

      # 2 if no, then end (wenn nein dann ende)
      $end=1;
    }
  }until($end);

  # as grouprid we use hex value of gid
  #$_userGroupRid = sprintf("%x", $_userGidNumber);
  #$_userRid  = sprintf("%x", $_userUidNumber);

  #if sombody wants the HEX value, who could it be?
  my $hex_uid= sprintf("%x", $uid);

  if($uid>=$end_uid){
    &error('0114',$uid);# ERROR: No UID left on device CipUX LDAP
    exit(0);
  }else{
    &log($SCRIPT,"- UID GIVEN: $uid ($hex_uid)");
    chomp($uid);
    chomp($hex_uid);

    &log($SCRIPT,"=> return \$uid \$hex_uid [$uid] [$hex_uid]");
    return $uid,$hex_uid;
  }

  &log($SCRIPT,"=> return [0]");
  return 0;# newert

}
# +========================================================================+
# ||                      cibot_get_uid_list                              ||
# +========================================================================+


=pod

=head2 cibot_get_uid_list

Return list of uid values (in a hash reference) which are present in
the LDAP server for trees: user, course and machine.

B<Syntax:>

  my $uid_list_ref = &cibot_get_uid_list($user_uid_list_command1,...);

B<Discussion:>

  @user_uid_list:

  user[TAB]uid

  get_value.pl -e uidNumber -U
  get_value.pl -e uidNumber -C
  get_value.pl -e uidNumber -M

=cut

sub cibot_get_uid_list{

  my $commands_ref = shift;# array ref

  &log($SCRIPT,"&cibot_get_uid_list($commands_ref)");

  my %uid_list=();

  foreach my $command (@$commands_ref){

    # META write error codes!
    &log($SCRIPT,"EXEC: $command");
    my @user_tab_uid_list = `$command`;

    foreach my $line (@user_tab_uid_list){
      my($object,$uid)  = split("\t",$line);
      chomp($uid);
      &log($SCRIPT,"OBJ $object UID '$uid'");
      $uid_list{"$uid"} = '1';
    }

    if($DEBUG){
      foreach my $uid (sort keys %uid_list){
	&log($SCRIPT,"UID LIST: $uid");
      }
    }
  }

  &log($SCRIPT,"=> return hash_ref uid_list [\%uid_list]");
  return \%uid_list;# hash_ref

}

# +========================================================================+
# ||                            get_password                              ||
# +========================================================================+

=pod

=head2 get_password

Makes passwords from a given string.
It will use the programm mkntpwd.
Return values are  CRYPT, lmPassword and ntPaswwort.

B<Syntax:>

  my($crypt,$lmPassword,$ntPaswwort) = &get_password($password);

=cut

sub get_password{

  #environment:
  my $password=shift;

  # MD5 password
 #my $random = join '',('.','/',0..9,'A'..'Z','a'..'z')[rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64,
 #						rand 64];
 #
 #my $bsalt = "\$1\$";
 #my $esalt = "\$";
 #my $modsalt = $bsalt.$random.$esalt;

  # crypt password
  my $modsalt = join '', ('.','/',0..9,'A'..'Z','a'..'z')[rand 64,rand 64];
  my $crypt_password = crypt($password, $modsalt);

  # LanManager and NT clear text passwords
  my $ntpwd = `/usr/local/sbin/mkntpwd '$password'`;
  chomp(my $lm_password = substr($ntpwd, 0, index($ntpwd, ':')));
  chomp(my $nt_password = substr($ntpwd, index($ntpwd, ':')+1));

  &log("Crypt password = '$crypt_password'");
  &log("NT passwort    = '$nt_password'");
  &log("LM passwort    = '$lm_password'");

  return "$crypt_password",$lm_password,$nt_password;

}









1;

__END__

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>

See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
