#!/usr/bin/perl -w
# +========================================================================+
# || cipux_internet_access                                                   ||
# ||                                                                      ||
# ||                                                                      ||
# ||                                                                      ||
# ||                                                                      ||
# || Copyright: 2005 by Christian Kuelker                                 ||
# || License: GPL Version 2 or later                                      ||
# +========================================================================+
#   MH 13.3.05  replaced 2 iptables calls with only one
#               see /etc/init.d/outblock
#               cipuxLocalnet and SquidPort are not needed anymore

=pod

=head1 NAME

cipux_internet_access - Version 1.0 2005-03-07  CipUX

=head1 SYNOPSIS

	cipux_internet_access [MANDATORY OPTIONS] -c COURSENAME

        cipux_internet_access -h

	Mandatory options: (only one possible)
        (01)  -d            :disable
        (02)  -e            :enable

        Other options:
        (03)  -h            :get help message


=head1 ABSTRACT

This script disables or enables the internet access for one course.

=head1 REQUIREMENTS

=cut


package cipux_internet_access;

# +========================================================================+
# |                           Global Part                                  |
# +========================================================================+

my $BRAND    = "cipworx";
my $PROJECT  = "cipux";
my $SCRIPT   = "cipux_internet_access";
my $VERSION  = "Wed Mar  9 02:40:37 CET 2005";
my $BUILD    = "<[CIBOT::BUILD]>";
my $Customer = "skolelinux";
my %opt;

&log($SCRIPT,"="x76);
&log($SCRIPT,"- Customer [$Customer]");

# +========================================================================+
# |                           Module                                       |
# +========================================================================+

# Load all modules
use strict;
use Getopt::Std;
use CipUX::CiBot;

# +========================================================================+
# ||                      Environment                                     ||
# +========================================================================+

my $options="edc:h";
&log($SCRIPT,"- Options: '$options'");

getopts("$options",\%opt);

# help
if(defined($opt{'h'})){&usage;exit(0);}

# no course
if(not defined($opt{'c'})){
  &usage;error('0193');#error::0193=The option -c is mandatory, please specify -c
}
if(not defined($opt{'e'}) and not defined($opt{'d'})){
  &usage;error('0194');#error::0194=One of the following options are mandatory: -e or -d
}

my $course = $opt{'c'};
my @user = `/usr/bin/cipux_get_value -c '$course' -e memberUid`;

if(defined($opt{'e'})){# ENABLE
  &log($SCRIPT,"* enable internet for course [$course]");
  # print "enable internet for course '$course'\n";
  print "erlaube Internet fuer die Gruppe: '$course'\n";
  foreach my $u (@user){
    chomp($u);
    &log($SCRIPT,"- enable internet for user [$u]"); 
    #    print "and therefore enable internet for user '$u'\n";
    print "und deswegen erlaube auch Internet fuer den Nutzer: '$u'\n";
    my $cmd = "iptables -D OUTPUT -m owner --uid-owner $u -j outblock";
    &log($SCRIPT,"- cmd [$cmd]");
    &cibot_exec_cmd('html',$cmd,"");
  $cmd = "/usr/bin/cipux_set_value -c '$course' -e cipuxInternetStatus -v accept";
  &log($SCRIPT,"- cmd =[$cmd]");
  &cibot_exec_cmd('html',$cmd,"");
  }
}else{# DISABLE
  &log($SCRIPT,"* disable internet for course [$course]");
  print "disable internet for course '$course'\n";
  print "verbiete Internet fuer die Gruppe: '$course'\n";
  foreach my $u (@user){
    chomp($u);
    &log($SCRIPT,"- disable internet for user [$u]"); 
    #    print "and therefore enable internet for user '$u'\n";
    print "und deswegen verbiete auch Internet fuer den Nutzer: '$u'\n";
    my $cmd = "iptables -I OUTPUT -m owner --uid-owner $u -j outblock";
    &log($SCRIPT,"- cmd [$cmd]");
    &cibot_exec_cmd('html',$cmd,"");
  $cmd = "/usr/bin/cipux_set_value -c '$course' -e cipuxInternetStatus -v reject";
  &log($SCRIPT,"- cmd =[$cmd]");
  &cibot_exec_cmd('html',$cmd,"");
  }
}

exit(0);

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

print "-"x76;
print "\n";
print "USAGE: ($BRAND-$PROJECT-$SCRIPT-$VERSION)\n\n";
print "       cipux_internet_access [MANDATORY OPTIONS] -c COURSENAME\n";
print "\n";
print "       cipux_internet_access -h\n";
print "\n";
print "       Mandatory options: (only one possible)\n";
print "       (01)  -e            :disable\n";
print "       (02)  -d            :enable\n";
print "\n";
print "       Other options:\n";
print "       (03)  -h            :get help message\n";
print "-"x76;
print "\n";
}



