#!/usr/bin/perl -w
# *========================================================================+
# || cipux_set_password                                                      ||
# || Version <[CIBOT::VERSION]>                                                    ||
# || This script sets a password for a given user, machine or course.     ||
# || It will use set_value.pl to fullfill the task.                       ||
# ||                                                                      ||
# ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!
# || Licence:   GPLv2
# || 
# || Revision:  2003-08-14
# || Build:     "<[CIBOT::BUILD]>";
# ||
# *========================================================================+

=pod

=head1 NAME

cipux_set_password - Version <[CIBOT::VERSION]>  2003-02-22 CipUX

=head1 SYNOPSIS

	cipux_set_password [OPTIONS]

	Auxiliary options: (one nessessary)
        (1)  -u        :sets a user password
        (2)  -c        :sets a course password (not implemented yet)
        (3)  -m        :sets a machine password (not implemented yet)

        Other options:
        (4)  -p        :password
        (5)  -h        :get help message

=head1 ABSTRACT

This script sets a password for a given user, machine or course.
It will use set_value.pl to fullfill the task. If option -p is
given, it will be used to set the password, otherwise the password
will be read from standard input.

=head1 REQUIREMENTS

       cibot.pm
       set_value.pl
       mkntpwd

=cut

package cipux_set_password;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_set_password";
my $VERSION = "<[CIBOT::VERSION]>";
my $Customer = "skolelinux";


&log($SCRIPT,"="x76);
# +========================================================================+
# |                           Module                                       |
# +========================================================================+

# Load all modules
use strict;
use CipUX::CiBot;

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();
use Getopt::Std;
getopts("p:u:c:m:h",\%opt);

if(defined($opt{h})){usage();exit(0)}

my %test=();
$test{u}=1 if defined($opt{u});
$test{c}=1 if defined($opt{c});
$test{m}=1 if defined($opt{m});

my $c=0;
foreach (sort keys %test){$c++}
# cipux_set_password: No auxiliary option -u, -c, or -m  given
if($c==0){&usage;&error('0095')}
# ok
if($c==1){&log($SCRIPT,"Got one option")}  
# cipux_set_password: To many options, need only one auxiliary option
if($c>1){&usage;&error('0096')}

my $search_node="";
$search_node="-u $opt{u}" if defined($opt{u});
$search_node="-c $opt{c}" if defined($opt{c});
$search_node="-m $opt{m}" if defined($opt{m});

my $password ="";
my $password2="";
# if no option p was given: input on STDIN
if(!defined($opt{p})){
  #print text('msg00025');# Please enter password:
  print "Please enter password:";
  #print "1\n" if $VERBOSE;
  system "stty -echo";
  chomp($password=<STDIN>);
  print "\n";
  system "stty echo";
  #print "2\n" if $VERBOSE;
  #print text('msg00026');# Please repeat password once again:
  print "Please repeat password once again:";
  system "stty -echo";
  chomp($password2=<STDIN>);
  print "\n";
  system "stty echo";
}else{
  # got password from -p 
  $password  = $opt{p};
  $password2 = $opt{p};
}

my $lm_password = "";
my $nt_password = "";
my $crypt = "";

if($password ne $password2) {
  #print text('msg00000')."\n";# ----
#  print text('msg00027')."\n";# Passwords do NOT match, exiting!
  print "Passwords do NOT match, exiting!\n";
  #print text('msg00000')."\n";# ----
  exit(1); # shell false
} else {

  if($password eq "") {
    #print text('msg00000')."\n";# ----
    #print text('msg00028')."\n";#Empty password given, rejected for security reason!!!
print "Empty password given, rejected for security reason!!!\n";
    #print text('msg00000')."\n";# ----
    exit(1); # shell false
  }
 # password
 #my()=&get_password();
 ($crypt,$lm_password,$nt_password) = &cibot_get_password($password);
  &log($SCRIPT,"- pwd: $crypt,$lm_password,$nt_password");
}

# +========================================================================+
$PART=                       "Configuration";#                             |
# +========================================================================+

#set_value.pl [OPTIONS] -e <ENTRY> -p <PASSWORD> -v <VALUE> -u/c/m XXX
# -e lmPassword
# -e ntPassword
# -e userPassword

my $cmd = "<[CIBOT::DIR]>/bin/set_value.pl $search_node";
my @commands=();
push(@commands,"$cmd -e 'lmPassword' -v '$lm_password'");
push(@commands,"$cmd -e 'ntPassword' -v '$nt_password'");
push(@commands,"$cmd -e 'userPassword' -v '{crypt}$crypt'");#{crypt}

&log($SCRIPT,"cibot::variable = 'set_value.pl'");
&log($SCRIPT,"cibot::variable = '<[CIBOT::DIR]>'");

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

# do the commands
foreach my $command (@commands){
  &log($SCRIPT,"- cmd: [$command]");
  my $pid=open(LDAP,"$command |") 
    or &error('0097'); # cipux_set_password: Can't execute LDAP search command
  my @output=<LDAP>;	
  my $ok=close(LDAP);
  error('0098') if !defined($pid); # cipux_set_password: Error occured while executing LDAP search (no PID)
  error('0099') if !$ok; # cipux_set_password: Error occured wile executing LDAP search (no bind or empty search)
}

exit(0); # shell ok

# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

#============================================================|

print "-"x76;
print "\n";
print "USAGE: ($BRAND-$PROJECT-$SCRIPT-$VERSION)\n";
print "       $SCRIPT [OPTIONS]\n";
print "Mandatory options:\n";
print "(1)  -p        :new password\n";
print "One required option:\n";
print "(2)  -u        :set password for user\n";
print "(3)  -c        :set password for course\n";
print "(4)  -m        :set password for machine\n";
print "Auxiliary options: (need only one)\n";
print "(5)  -h        :this message\n";
print "-"x76;
print "\n";
}

=pod

=head1 AUTOR

Author Christian Kuelker <christian.kuelker@cipworx.org>

Copyright (c) 2003-02-22 Christian Kuelker

Published under the GNU Public License: GPLv2

=head1 BUGS

Not known.

=head1 SEE ALSO

CiBot Manual, set_value.pl

=cut

